@@ -1,4 +1,4 @@
-/* $Id: array.h,v 1.9 2009/11/02 21:34:32 tcunha Exp $ */
+/* $Id: array.h,v 1.11 2010/06/06 00:27:08 tcunha Exp $ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -19,6 +19,8 @@
 #ifndef ARRAY_H
 #define ARRAY_H
 
+#define ARRAY_INITIALIZER { NULL, 0, 0 }
+
 #define ARRAY_DECL(n, c)						\
 	struct n {							\
 		c	*list;						\
@@ -45,7 +47,7 @@
 	}								\
 } while (0)
 
-#define ARRAY_EMPTY(a) ((a) == NULL || (a)->num == 0)
+#define ARRAY_EMPTY(a) (((void *) (a)) == NULL || (a)->num == 0)
 #define ARRAY_LENGTH(a) ((a)->num)
 #define ARRAY_DATA(a) ((a)->list)
 
