@@ -1,4 +1,4 @@
-/* $Id: client.c,v 1.84 2009/11/02 21:41:16 tcunha Exp $ */
+/* $Id: client.c,v 1.95 2010/07/02 02:52:13 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,13 +17,13 @@
  */
 
 #include <sys/types.h>
-#include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
 #include <sys/wait.h>
 
 #include <errno.h>
+#include <event.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <stdlib.h>
@@ -34,13 +34,17 @@
 #include "tmux.h"
 
 struct imsgbuf	client_ibuf;
+struct event	client_event;
 const char     *client_exitmsg;
+int		client_exitval;
 
-void	client_send_identify(int);
-void	client_send_environ(void);
-void	client_write_server(enum msgtype, void *, size_t);
-int	client_dispatch(void);
-void	client_suspend(void);
+void		client_send_identify(int);
+void		client_send_environ(void);
+void		client_write_server(enum msgtype, void *, size_t);
+void		client_update_event(void);
+void		client_signal(int, short, void *);
+void		client_callback(int, short, void *);
+int		client_dispatch(void);
 
 struct imsgbuf *
 client_init(char *path, int cmdflags, int flags)
@@ -96,8 +100,7 @@ server_started:
 
 	if (cmdflags & CMD_SENDENVIRON)
 		client_send_environ();
-	if (isatty(STDIN_FILENO))
-		client_send_identify(flags);
+	client_send_identify(flags);
 
 	return (&client_ibuf);
 
@@ -114,17 +117,14 @@ void
 client_send_identify(int flags)
 {
 	struct msg_identify_data	data;
-	struct winsize			ws;
 	char			       *term;
 	int				fd;
 
-	if (ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) == -1)
-		fatal("ioctl(TIOCGWINSZ)");
 	data.flags = flags;
 
 	if (getcwd(data.cwd, sizeof data.cwd) == NULL)
 		*data.cwd = '\0';
-	
+
 	term = getenv("TERM");
 	if (term == NULL ||
 	    strlcpy(data.term, term, sizeof data.term) >= sizeof data.term)
@@ -134,6 +134,14 @@ client_send_identify(int flags)
 		fatal("dup failed");
 	imsg_compose(&client_ibuf,
 	    MSG_IDENTIFY, PROTOCOL_VERSION, -1, fd, &data, sizeof data);
+
+	if ((fd = dup(STDOUT_FILENO)) == -1)
+		fatal("dup failed");
+	imsg_compose(&client_ibuf, MSG_STDOUT, PROTOCOL_VERSION, -1, fd, NULL, 0);
+
+	if ((fd = dup(STDERR_FILENO)) == -1)
+		fatal("dup failed");
+	imsg_compose(&client_ibuf, MSG_STDERR, PROTOCOL_VERSION, -1, fd, NULL, 0);
 }
 
 void
@@ -142,7 +150,7 @@ client_send_environ(void)
 	struct msg_environ_data	data;
 	char		      **var;
 
- 	for (var = environ; *var != NULL; var++) {
+	for (var = environ; *var != NULL; var++) {
 		if (strlcpy(data.var, *var, sizeof data.var) >= sizeof data.var)
 			continue;
 		client_write_server(MSG_ENVIRON, &data, sizeof data);
@@ -152,99 +160,129 @@ client_send_environ(void)
 void
 client_write_server(enum msgtype type, void *buf, size_t len)
 {
- 	imsg_compose(&client_ibuf, type, PROTOCOL_VERSION, -1, -1, buf, len);
+	imsg_compose(&client_ibuf, type, PROTOCOL_VERSION, -1, -1, buf, len);
+}
+
+void
+client_update_event(void)
+{
+	short	events;
+
+	event_del(&client_event);
+	events = EV_READ;
+	if (client_ibuf.w.queued > 0)
+		events |= EV_WRITE;
+	event_set(&client_event, client_ibuf.fd, events, client_callback, NULL);
+	event_add(&client_event, NULL);
 }
 
 __dead void
 client_main(void)
 {
-	struct pollfd	 pfd;
-	int		 n, nfds;
-
-	siginit();
-
 	logfile("client");
 
+	/* Note: event_init() has already been called. */
+
+	/* Set up signals. */
+	set_signals(client_signal);
+
+	/*
+	 * Send a resize message immediately in case the terminal size has
+	 * changed between the identify message to the server and the MSG_READY
+	 * telling us to move into the client code.
+	 */
+	 client_write_server(MSG_RESIZE, NULL, 0);
+
 	/*
 	 * imsg_read in the first client poll loop (before the terminal has
-	 * been initialiased) may have read messages into the buffer after the
-	 * MSG_READY switched to here. Process anything outstanding now so poll
-	 * doesn't hang waiting for messages that have already arrived.
+	 * been initialised) may have read messages into the buffer after the
+	 * MSG_READY switched to here. Process anything outstanding now to
+	 * avoid hanging waiting for messages that have already arrived.
 	 */
 	if (client_dispatch() != 0)
 		goto out;
 
-	for (;;) {
-		if (sigterm) {
-			client_exitmsg = "terminated";
-			client_write_server(MSG_EXITING, NULL, 0);
-		}
-		if (sigchld) {
-			sigchld = 0;
-			waitpid(WAIT_ANY, NULL, WNOHANG);
-			continue;
-		}
-		if (sigwinch) {
-			sigwinch = 0;
-			client_write_server(MSG_RESIZE, NULL, 0);
-			continue;
-		}
-		if (sigcont) {
-			sigcont = 0;
-			siginit();
-			client_write_server(MSG_WAKEUP, NULL, 0);
-			continue;
-		}
-
-		pfd.fd = client_ibuf.fd;
-		pfd.events = POLLIN;
-		if (client_ibuf.w.queued > 0)
-			pfd.events |= POLLOUT;
-
-		if ((nfds = poll(&pfd, 1, INFTIM)) == -1) {
-			if (errno == EAGAIN || errno == EINTR)
-				continue;
-			fatal("poll failed");
-		}
-		if (nfds == 0)
-			continue;
-
-		if (pfd.revents & (POLLERR|POLLHUP|POLLNVAL))
-			fatalx("socket error");
-
-		if (pfd.revents & POLLIN) {
-			if ((n = imsg_read(&client_ibuf)) == -1 || n == 0) {
-				client_exitmsg = "lost server";
-				break;
-			}
-			if (client_dispatch() != 0)
-				break;
-		}
-
-		if (pfd.revents & POLLOUT) {
-			if (msgbuf_write(&client_ibuf.w) < 0) {
-				client_exitmsg = "lost server";
-				break;
-			}
-		}
-	}
+	/* Set the event and dispatch. */
+	client_update_event();
+	event_dispatch();
 
 out:
 	/* Print the exit message, if any, and exit. */
-	if (client_exitmsg != NULL) {
-		if (!login_shell)
-			printf("[%s]\n", client_exitmsg);
-		exit(1);
+	if (client_exitmsg != NULL && !login_shell)
+		printf("[%s]\n", client_exitmsg);
+	exit(client_exitval);
+}
+
+/* ARGSUSED */
+void
+client_signal(int sig, unused short events, unused void *data)
+{
+	struct sigaction	sigact;
+
+	switch (sig) {
+	case SIGHUP:
+		client_exitmsg = "lost tty";
+		client_exitval = 1;
+		client_write_server(MSG_EXITING, NULL, 0);
+		break;
+	case SIGTERM:
+		client_exitmsg = "terminated";
+		client_exitval = 1;
+		client_write_server(MSG_EXITING, NULL, 0);
+		break;
+	case SIGWINCH:
+		client_write_server(MSG_RESIZE, NULL, 0);
+		break;
+	case SIGCONT:
+		memset(&sigact, 0, sizeof sigact);
+		sigemptyset(&sigact.sa_mask);
+		sigact.sa_flags = SA_RESTART;
+		sigact.sa_handler = SIG_IGN;
+		if (sigaction(SIGTSTP, &sigact, NULL) != 0)
+			fatal("sigaction failed");
+		client_write_server(MSG_WAKEUP, NULL, 0);
+		break;
 	}
-	exit(0);
+
+	client_update_event();
+}
+
+/* ARGSUSED */
+void
+client_callback(unused int fd, short events, unused void *data)
+{
+	ssize_t	n;
+
+	if (events & EV_READ) {
+		if ((n = imsg_read(&client_ibuf)) == -1 || n == 0)
+			goto lost_server;
+		if (client_dispatch() != 0) {
+			event_loopexit(NULL);
+			return;
+		}
+	}
+
+	if (events & EV_WRITE) {
+		if (msgbuf_write(&client_ibuf.w) < 0)
+			goto lost_server;
+	}
+
+	client_update_event();
+	return;
+
+lost_server:
+	client_exitmsg = "lost server";
+	client_exitval = 1;
+	event_loopexit(NULL);
 }
 
 int
 client_dispatch(void)
 {
-	struct imsg		 imsg;
-	struct msg_lock_data	 lockdata;
-	ssize_t			 n, datalen;
+	struct imsg		imsg;
+	struct msg_lock_data	lockdata;
+	struct sigaction	sigact;
+	ssize_t			n, datalen;
 
 	for (;;) {
 		if ((n = imsg_get(&client_ibuf, &imsg)) == -1)
@@ -253,6 +291,7 @@ client_dispatch(void)
 			return (0);
 		datalen = imsg.hdr.len - IMSG_HEADER_SIZE;
 
+		log_debug("client got %d", imsg.hdr.type);
 		switch (imsg.hdr.type) {
 		case MSG_DETACH:
 			if (datalen != 0)
@@ -280,18 +319,25 @@ client_dispatch(void)
 
 			client_write_server(MSG_EXITING, NULL, 0);
 			client_exitmsg = "server exited";
+			client_exitval = 1;
 			break;
 		case MSG_SUSPEND:
 			if (datalen != 0)
 				fatalx("bad MSG_SUSPEND size");
 
-			client_suspend();
+			memset(&sigact, 0, sizeof sigact);
+			sigemptyset(&sigact.sa_mask);
+			sigact.sa_flags = SA_RESTART;
+			sigact.sa_handler = SIG_DFL;
+			if (sigaction(SIGTSTP, &sigact, NULL) != 0)
+				fatal("sigaction failed");
+			kill(getpid(), SIGTSTP);
 			break;
 		case MSG_LOCK:
 			if (datalen != sizeof lockdata)
 				fatalx("bad MSG_LOCK size");
 			memcpy(&lockdata, imsg.data, sizeof lockdata);
-			
+
 			lockdata.cmd[(sizeof lockdata.cmd) - 1] = '\0';
 			system(lockdata.cmd);
 			client_write_server(MSG_UNLOCK, NULL, 0);
@@ -303,23 +349,3 @@ client_dispatch(void)
 		imsg_free(&imsg);
 	}
 }
-
-void
-client_suspend(void)
-{
-	struct sigaction	 act;
-
-	memset(&act, 0, sizeof act);
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = SA_RESTART;
-
-	act.sa_handler = SIG_DFL;
-	if (sigaction(SIGTSTP, &act, NULL) != 0)
-		fatal("sigaction failed");
-
-	act.sa_handler = sighandler;
-	if (sigaction(SIGCONT, &act, NULL) != 0)
-		fatal("sigaction failed");
-
-	kill(getpid(), SIGTSTP);
-}
