@@ -1,4 +1,4 @@
-/* $Id: clock.c,v 1.7 2009/09/11 14:13:52 tcunha Exp $ */
+/* $Id: clock.c,v 1.9 2009/12/04 22:14:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -97,7 +97,7 @@ const char clock_table[14][5][5] = {
 };
 
 void
-clock_draw(struct screen_write_ctx *ctx, u_int colour, int style)
+clock_draw(struct screen_write_ctx *ctx, int colour, int style)
 {
 	struct screen		*s = ctx->s;
 	struct grid_cell	 gc;
@@ -134,13 +134,13 @@ clock_draw(struct screen_write_ctx *ctx, u_int colour, int style)
 	for (ptr = tim; *ptr != '\0'; ptr++) {
 		if (*ptr >= '0' && *ptr <= '9')
 			idx = *ptr - '0';
- 		else if (*ptr == ':')
+		else if (*ptr == ':')
 			idx = 10;
- 		else if (*ptr == 'A')
+		else if (*ptr == 'A')
 			idx = 11;
- 		else if (*ptr == 'P')
+		else if (*ptr == 'P')
 			idx = 12;
- 		else if (*ptr == 'M')
+		else if (*ptr == 'M')
 			idx = 13;
 		else {
 			x += 6;
