@@ -1,4 +1,4 @@
-/* $Id: cmd-attach-session.c,v 1.32 2009/08/09 17:48:55 tcunha Exp $ */
+/* $Id: cmd-attach-session.c,v 1.36 2010/02/08 18:27:34 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,8 +28,8 @@ int	cmd_attach_session_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_attach_session_entry = {
 	"attach-session", "attach",
-	"[-d] " CMD_TARGET_SESSION_USAGE,
-       	CMD_CANTNEST|CMD_STARTSERVER|CMD_SENDENVIRON, CMD_CHFLAG('d'),
+	"[-dr] " CMD_TARGET_SESSION_USAGE,
+	CMD_CANTNEST|CMD_STARTSERVER|CMD_SENDENVIRON, "dr",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_attach_session_exec,
@@ -58,8 +58,8 @@ cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return (0);
 
 	if (ctx->cmdclient == NULL) {
-		if (data->chflags & CMD_CHFLAG('d')) {
-			/* 
+		if (cmd_check_flag(data->chflags, 'd')) {
+			/*
 			 * Can't use server_write_session in case attaching to
 			 * the same session as currently attached to.
 			 */
@@ -72,7 +72,7 @@ cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 				server_write_client(c, MSG_DETACH, NULL, 0);
 			}
 		}
-		
+
 		ctx->curclient->session = s;
 		server_redraw_client(ctx->curclient);
 	} else {
@@ -89,7 +89,10 @@ cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 			return (-1);
 		}
 
-		if (data->chflags & CMD_CHFLAG('d'))
+		if (cmd_check_flag(data->chflags, 'r'))
+			ctx->cmdclient->flags |= CLIENT_READONLY;
+
+		if (cmd_check_flag(data->chflags, 'd'))
 			server_write_session(s, MSG_DETACH, NULL, 0);
 
 		ctx->cmdclient->session = s;
@@ -101,6 +104,7 @@ cmd_attach_session_exec(struct cmd *self, struct cmd_ctx *ctx)
 		server_redraw_client(ctx->cmdclient);
 	}
 	recalculate_sizes();
+	server_update_socket();
 
 	return (1);	/* 1 means don't tell command client to exit */
 }
