@@ -1,4 +1,4 @@
-/* $Id: cmd-command-prompt.c,v 1.26 2009/09/22 14:06:40 tcunha Exp $ */
+/* $Id: cmd-command-prompt.c,v 1.28 2010/05/14 14:33:39 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -39,7 +39,7 @@ void	 cmd_command_prompt_cfree(void *);
 const struct cmd_entry cmd_command_prompt_entry = {
 	"command-prompt", NULL,
 	CMD_TARGET_CLIENT_USAGE " [-p prompts] [template]",
-	0, 0,
+	0, "",
 	cmd_command_prompt_init,
 	cmd_command_prompt_parse,
 	cmd_command_prompt_exec,
@@ -81,6 +81,10 @@ cmd_command_prompt_init(struct cmd *self, int key)
 	case 'f':
 		data->template = xstrdup("find-window '%%'");
 		break;
+	case '\'':
+		data->template = xstrdup("select-window -t ':%%'");
+		data->prompts = xstrdup("index");
+		break;
 	}
 }
 
