@@ -1,4 +1,4 @@
-/* $Id: cmd-copy-buffer.c,v 1.5 2009/09/22 14:06:40 tcunha Exp $ */
+/* $Id: cmd-copy-buffer.c,v 1.7 2009/11/28 14:50:36 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Tiago Cunha <me@tiagocunha.org>
@@ -43,7 +43,7 @@ struct cmd_copy_buffer_data {
 const struct cmd_entry cmd_copy_buffer_entry = {
 	"copy-buffer", "copyb",
 	"[-a src-index] [-b dst-index] [-s src-session] [-t dst-session]",
-	0, 0,
+	0, "",
 	cmd_copy_buffer_init,
 	cmd_copy_buffer_parse,
 	cmd_copy_buffer_exec,
@@ -51,6 +51,7 @@ const struct cmd_entry cmd_copy_buffer_entry = {
 	cmd_copy_buffer_print
 };
 
+/* ARGSUSED */
 void
 cmd_copy_buffer_init(struct cmd *self, unused int arg)
 {
