@@ -1,4 +1,4 @@
-/* $Id: cmd-generic.c,v 1.34 2009/08/26 22:13:52 tcunha Exp $ */
+/* $Id: cmd-generic.c,v 1.38 2009/12/04 22:14:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -23,8 +23,8 @@
 
 #include "tmux.h"
 
-int	cmd_getopt(int, char **, const char *, uint64_t);
-int	cmd_flags(int, uint64_t, uint64_t *);
+int	cmd_getopt(int, char **, const char *, const char *);
+int	cmd_parse_flags(int, const char *, uint64_t *);
 size_t	cmd_print_flags(char *, size_t, size_t, uint64_t);
 int	cmd_fill_argument(int, char **, char **, int, char **);
 
@@ -36,54 +36,56 @@ cmd_prarg(char *buf, size_t len, const char *prefix, char *arg)
 	return (xsnprintf(buf, len, "%s%s", prefix, arg));
 }
 
-/* Prepend flags from chflags onto flagstr and call getopt. */
+/* Append two flag strings together and call getopt. */
 int
-cmd_getopt(int argc, char **argv, const char *flagstr, uint64_t chflags)
+cmd_getopt(int argc, char **argv, const char *flagstr, const char *chflagstr)
 {
-	u_char	ch;
-	char	buf[128];
-	size_t	len, off;
+	char	tmp[BUFSIZ];
 
-	*buf = '\0';
-
-	len = sizeof buf;
-	off = 0;
-
-	for (ch = 0; ch < 26; ch++) {
-		if (chflags & CMD_CHFLAG('a' + ch))
-			off += xsnprintf(buf + off, len - off, "%c", 'a' + ch);
-		if (chflags & CMD_CHFLAG('A' + ch))
-			off += xsnprintf(buf + off, len - off, "%c", 'A' + ch);
-	}
-	
-	strlcat(buf, flagstr, sizeof buf);
-
-	return (getopt(argc, argv, buf));
+	if (strlcpy(tmp, flagstr, sizeof tmp) >= sizeof tmp)
+		fatalx("strlcpy overflow");
+	if (strlcat(tmp, chflagstr, sizeof tmp) >= sizeof tmp)
+		fatalx("strlcat overflow");
+	return (getopt(argc, argv, tmp));
 }
 
-/* 
- * If this option is expected (in ichflags), set it in ochflags, otherwise
- * return -1.
- */
+/* Return if flag character is set. */
 int
-cmd_flags(int opt, uint64_t ichflags, uint64_t *ochflags)
+cmd_check_flag(uint64_t chflags, int flag)
 {
-	u_char	ch;
-
-	for (ch = 0; ch < 26; ch++) {
-		if (opt == 'a' + ch && ichflags & CMD_CHFLAG(opt)) {
-			(*ochflags) |= CMD_CHFLAG(opt);
-			return (0);
-		}
-		if (opt == 'A' + ch && ichflags & CMD_CHFLAG(opt)) {
-			(*ochflags) |= CMD_CHFLAG(opt);
-			return (0);
-		}
-	}
-	return (-1);
+	if (flag >= 'A' && flag <= 'Z')
+		flag = 26 + flag - 'A';
+	else if (flag >= 'a' && flag <= 'z')
+		flag = flag - 'a';
+	else
+		return (0);
+	return ((chflags & (1ULL << flag)) != 0);
 }
 
-/* Print the flags supported in chflags. */
+/* Set flag character. */
+void
+cmd_set_flag(uint64_t *chflags, int flag)
+{
+	if (flag >= 'A' && flag <= 'Z')
+		flag = 26 + flag - 'A';
+	else if (flag >= 'a' && flag <= 'z')
+		flag = flag - 'a';
+	else
+		return;
+	(*chflags) |= (1ULL << flag);
+}
+
+/* If this option is expected, set it in chflags, otherwise return -1. */
+int
+cmd_parse_flags(int opt, const char *chflagstr, uint64_t *chflags)
+{
+	if (strchr(chflagstr, opt) == NULL)
+		return (-1);
+	cmd_set_flag(chflags, opt);
+	return (0);
+}
+
+/* Print the flags present in chflags. */
 size_t
 cmd_print_flags(char *buf, size_t len, size_t off, uint64_t chflags)
 {
@@ -95,9 +97,9 @@ cmd_print_flags(char *buf, size_t len, size_t off, uint64_t chflags)
 	off += xsnprintf(buf + off, len - off, " -");
 
 	for (ch = 0; ch < 26; ch++) {
-		if (chflags & CMD_CHFLAG('a' + ch))
+		if (cmd_check_flag(chflags, 'a' + ch))
 			off += xsnprintf(buf + off, len - off, "%c", 'a' + ch);
-		if (chflags & CMD_CHFLAG('A' + ch))
+		if (cmd_check_flag(chflags, 'A' + ch))
 			off += xsnprintf(buf + off, len - off, "%c", 'A' + ch);
 	}
 	return (off - boff);
@@ -139,13 +141,14 @@ cmd_fill_argument(int flags, char **arg, char **arg2, int argc, char **argv)
 		if (argc == 2)
 			*arg2 = xstrdup(argv[1]);
 		return (0);
-	}		
+	}
 
 	if (argc != 0)
 		return (-1);
 	return (0);
 }
 
+/* ARGSUSED */
 void
 cmd_target_init(struct cmd *self, unused int key)
 {
@@ -170,7 +173,7 @@ cmd_target_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data = self->data;
 
 	while ((opt = cmd_getopt(argc, argv, "t:", entry->chflags)) != -1) {
-		if (cmd_flags(opt, entry->chflags, &data->chflags) == 0)
+		if (cmd_parse_flags(opt, entry->chflags, &data->chflags) == 0)
 			continue;
 		switch (opt) {
 		case 't':
@@ -222,13 +225,14 @@ cmd_target_print(struct cmd *self, char *buf, size_t len)
 	off += cmd_print_flags(buf, len, off, data->chflags);
 	if (off < len && data->target != NULL)
 		off += cmd_prarg(buf + off, len - off, " -t ", data->target);
- 	if (off < len && data->arg != NULL)
+	if (off < len && data->arg != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg);
- 	if (off < len && data->arg2 != NULL)
+	if (off < len && data->arg2 != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg2);
 	return (off);
 }
 
+/* ARGSUSED */
 void
 cmd_srcdst_init(struct cmd *self, unused int key)
 {
@@ -253,7 +257,7 @@ cmd_srcdst_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data = self->data;
 
 	while ((opt = cmd_getopt(argc, argv, "s:t:", entry->chflags)) != -1) {
-		if (cmd_flags(opt, entry->chflags, &data->chflags) == 0)
+		if (cmd_parse_flags(opt, entry->chflags, &data->chflags) == 0)
 			continue;
 		switch (opt) {
 		case 's':
@@ -313,13 +317,14 @@ cmd_srcdst_print(struct cmd *self, char *buf, size_t len)
 		off += xsnprintf(buf + off, len - off, " -s %s", data->src);
 	if (off < len && data->dst != NULL)
 		off += xsnprintf(buf + off, len - off, " -t %s", data->dst);
- 	if (off < len && data->arg != NULL)
+	if (off < len && data->arg != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg);
- 	if (off < len && data->arg2 != NULL)
+	if (off < len && data->arg2 != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg2);
 	return (off);
 }
 
+/* ARGSUSED */
 void
 cmd_buffer_init(struct cmd *self, unused int key)
 {
@@ -345,7 +350,7 @@ cmd_buffer_parse(struct cmd *self, int argc, char **argv, char **cause)
 	data = self->data;
 
 	while ((opt = cmd_getopt(argc, argv, "b:t:", entry->chflags)) != -1) {
-		if (cmd_flags(opt, entry->chflags, &data->chflags) == 0)
+		if (cmd_parse_flags(opt, entry->chflags, &data->chflags) == 0)
 			continue;
 		switch (opt) {
 		case 'b':
@@ -410,9 +415,9 @@ cmd_buffer_print(struct cmd *self, char *buf, size_t len)
 		off += xsnprintf(buf + off, len - off, " -b %d", data->buffer);
 	if (off < len && data->target != NULL)
 		off += cmd_prarg(buf + off, len - off, " -t ", data->target);
- 	if (off < len && data->arg != NULL)
+	if (off < len && data->arg != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg);
- 	if (off < len && data->arg2 != NULL)
+	if (off < len && data->arg2 != NULL)
 		off += cmd_prarg(buf + off, len - off, " ", data->arg2);
 	return (off);
 }
