@@ -1,4 +1,4 @@
-/* $Id: cmd-if-shell.c,v 1.7 2009/11/02 21:38:26 tcunha Exp $ */
+/* $Id: cmd-if-shell.c,v 1.9 2010/07/17 14:36:40 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Tiago Cunha <me@tiagocunha.org>
@@ -36,7 +36,7 @@ void	cmd_if_shell_free(void *);
 const struct cmd_entry cmd_if_shell_entry = {
 	"if-shell", "if",
 	"shell-command command",
-	CMD_ARG2, 0,
+	CMD_ARG2, "",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_if_shell_exec,
@@ -104,10 +104,13 @@ cmd_if_shell_free(void *data)
 {
 	struct cmd_if_shell_data	*cdata = data;
 	struct cmd_ctx			*ctx = &cdata->ctx;
+	struct msg_exit_data		 exitdata;
 
 	if (ctx->cmdclient != NULL) {
 		ctx->cmdclient->references--;
-		server_write_client(ctx->cmdclient, MSG_EXIT, NULL, 0);
+		exitdata.retcode = ctx->cmdclient->retcode;
+		server_write_client(
+		    ctx->cmdclient, MSG_EXIT, &exitdata, sizeof exitdata);
 	}
 	if (ctx->curclient != NULL)
 		ctx->curclient->references--;
