@@ -1,4 +1,4 @@
-/* $Id: cmd-kill-server.c,v 1.8 2009/07/28 22:12:16 tcunha Exp $ */
+/* $Id: cmd-kill-server.c,v 1.11 2009/11/28 14:50:36 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -32,7 +32,7 @@ int	cmd_kill_server_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_kill_server_entry = {
 	"kill-server", NULL,
 	"",
-	0, 0,
+	0, "",
 	NULL,
 	NULL,
 	cmd_kill_server_exec,
@@ -40,10 +40,11 @@ const struct cmd_entry cmd_kill_server_entry = {
 	NULL
 };
 
+/* ARGSUSED */
 int
 cmd_kill_server_exec(unused struct cmd *self, unused struct cmd_ctx *ctx)
 {
-	sigterm = 1;
+	kill(getpid(), SIGTERM);
 
 	return (0);
 }
