@@ -1,4 +1,4 @@
-/* $Id: cmd-kill-window.c,v 1.20 2009/09/20 22:15:32 tcunha Exp $ */
+/* $Id: cmd-kill-window.c,v 1.21 2009/11/14 17:56:39 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,7 +29,7 @@ int	cmd_kill_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_kill_window_entry = {
 	"kill-window", "killw",
 	CMD_TARGET_WINDOW_USAGE,
-	0, 0,
+	0, "",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_kill_window_exec,
