@@ -1,4 +1,4 @@
-/* $Id: cmd-list-commands.c,v 1.5 2009/07/28 22:12:16 tcunha Exp $ */
+/* $Id: cmd-list-commands.c,v 1.7 2009/11/28 14:50:36 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,7 +29,7 @@ int	cmd_list_commands_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_list_commands_entry = {
 	"list-commands", "lscm",
 	"",
-	0, 0,
+	0, "",
 	NULL,
 	NULL,
 	cmd_list_commands_exec,
@@ -37,6 +37,7 @@ const struct cmd_entry cmd_list_commands_entry = {
 	NULL
 };
 
+/* ARGSUSED */
 int
 cmd_list_commands_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 {
