@@ -1,4 +1,4 @@
-/* $Id: cmd-list-windows.c,v 1.41 2009/10/12 00:08:12 tcunha Exp $ */
+/* $Id: cmd-list-windows.c,v 1.43 2010/07/02 02:54:52 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,7 +31,7 @@ int	cmd_list_windows_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_list_windows_entry = {
 	"list-windows", "lsw",
 	CMD_TARGET_SESSION_USAGE,
-	0, 0,
+	0, "",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_list_windows_exec,
@@ -45,6 +45,7 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct cmd_target_data	*data = self->data;
 	struct session		*s;
 	struct winlink		*wl;
+	char			*layout;
 
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
 		return (-1);
@@ -52,6 +53,9 @@ cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 	RB_FOREACH(wl, winlinks, &s->windows) {
 		ctx->print(ctx, "%d: %s [%ux%u]",
 		    wl->idx, wl->window->name, wl->window->sx, wl->window->sy);
+		layout = layout_dump(wl->window);
+		ctx->print(ctx, "    layout: %s", layout);
+		xfree(layout);
 	}
 
 	return (0);
