@@ -1,4 +1,4 @@
-/* $Id: cmd-new-window.c,v 1.39 2009/10/11 23:38:16 tcunha Exp $ */
+/* $Id: cmd-new-window.c,v 1.47 2010/07/02 02:49:19 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -36,14 +36,15 @@ struct cmd_new_window_data {
 	char	*target;
 	char	*name;
 	char	*cmd;
+	int	 flag_insert_after;
 	int	 flag_detached;
 	int	 flag_kill;
 };
 
 const struct cmd_entry cmd_new_window_entry = {
 	"new-window", "neww",
-	"[-dk] [-n window-name] [-t target-window] [command]",
-	0, 0,
+	"[-adk] [-n window-name] [-t target-window] [command]",
+	0, "",
 	cmd_new_window_init,
 	cmd_new_window_parse,
 	cmd_new_window_exec,
@@ -51,6 +52,7 @@ const struct cmd_entry cmd_new_window_entry = {
 	cmd_new_window_print
 };
 
+/* ARGSUSED */
 void
 cmd_new_window_init(struct cmd *self, unused int arg)
 {
@@ -60,6 +62,7 @@ cmd_new_window_init(struct cmd *self, unused int arg)
 	data->target = NULL;
 	data->name = NULL;
 	data->cmd = NULL;
+	data->flag_insert_after = 0;
 	data->flag_detached = 0;
 	data->flag_kill = 0;
 }
@@ -73,8 +76,11 @@ cmd_new_window_parse(struct cmd *self, int argc, char **argv, char **cause)
 	self->entry->init(self, KEYC_NONE);
 	data = self->data;
 
-	while ((opt = getopt(argc, argv, "dkt:n:")) != -1) {
+	while ((opt = getopt(argc, argv, "adkt:n:")) != -1) {
 		switch (opt) {
+		case 'a':
+			data->flag_insert_after = 1;
+			break;
 		case 'd':
 			data->flag_detached = 1;
 			break;
@@ -117,42 +123,66 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session			*s;
 	struct winlink			*wl;
 	char				*cmd, *cwd, *cause;
-	int				 idx;
+	int				 idx, last;
 
 	if (data == NULL)
 		return (0);
 
-	if ((idx = cmd_find_index(ctx, data->target, &s)) == -2)
-		return (-1);
+	if (data->flag_insert_after) {
+		if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
+			return (-1);
+		idx = wl->idx + 1;
+
+		/* Find the next free index. */
+		for (last = idx; last < INT_MAX; last++) {
+			if (winlink_find_by_index(&s->windows, last) == NULL)
+				break;
+		}
+		if (last == INT_MAX) {
+			ctx->error(ctx, "no free window indexes");
+			return (-1);
+		}
+
+		/* Move everything from last - 1 to idx up a bit. */
+		for (; last > idx; last--) {
+			wl = winlink_find_by_index(&s->windows, last - 1);
+			server_link_window(s, wl, s, last, 0, 0, NULL);
+			server_unlink_window(s, wl);
+		}
+	} else {
+		if ((idx = cmd_find_index(ctx, data->target, &s)) == -2)
+			return (-1);
+	}
 
 	wl = NULL;
 	if (idx != -1)
 		wl = winlink_find_by_index(&s->windows, idx);
-	if (wl != NULL) {
-		if (data->flag_kill) {
-			/*
-			 * Can't use session_detach as it will destroy session
-			 * if this makes it empty.
-			 */
-			session_alert_cancel(s, wl);
-			winlink_stack_remove(&s->lastw, wl);
-			winlink_remove(&s->windows, wl);
+	if (wl != NULL && data->flag_kill) {
+		/*
+		 * Can't use session_detach as it will destroy session if this
+		 * makes it empty.
+		 */
+		wl->flags &= ~WINLINK_ALERTFLAGS;
+		winlink_stack_remove(&s->lastw, wl);
+		winlink_remove(&s->windows, wl);
 
-			/* Force select/redraw if current. */
-			if (wl == s->curw) {
-				data->flag_detached = 0;
-				s->curw = NULL;
-			}
+		/* Force select/redraw if current. */
+		if (wl == s->curw) {
+			data->flag_detached = 0;
+			s->curw = NULL;
 		}
 	}
 
 	cmd = data->cmd;
 	if (cmd == NULL)
 		cmd = options_get_string(&s->options, "default-command");
-	if (ctx->cmdclient == NULL || ctx->cmdclient->cwd == NULL)
-		cwd = options_get_string(&s->options, "default-path");
-	else
-		cwd = ctx->cmdclient->cwd;
+	cwd = options_get_string(&s->options, "default-path");
+	if (*cwd == '\0') {
+		if (ctx->cmdclient != NULL && ctx->cmdclient->cwd != NULL)
+			cwd = ctx->cmdclient->cwd;
+		else
+			cwd = s->cwd;
+	}
 
 	if (idx == -1)
 		idx = -1 - options_get_number(&s->options, "base-index");
@@ -165,7 +195,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	if (!data->flag_detached) {
 		session_select(s, wl->idx);
 		server_redraw_session_group(s);
-	} else	
+	} else
 		server_status_session_group(s);
 
 	return (0);
