@@ -1,4 +1,4 @@
-/* $Id: cmd-pipe-pane.c,v 1.3 2009/10/23 17:26:40 tcunha Exp $ */
+/* $Id: cmd-pipe-pane.c,v 1.13 2010/06/15 20:25:40 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -17,10 +17,12 @@
  */
 
 #include <sys/types.h>
+#include <sys/socket.h>
 
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "tmux.h"
@@ -31,10 +33,12 @@
 
 int	cmd_pipe_pane_exec(struct cmd *, struct cmd_ctx *);
 
+void	cmd_pipe_pane_error_callback(struct bufferevent *, short, void *);
+
 const struct cmd_entry cmd_pipe_pane_entry = {
 	"pipe-pane", "pipep",
 	CMD_TARGET_PANE_USAGE "[-o] [command]",
-	CMD_ARG01, CMD_CHFLAG('o'),
+	CMD_ARG01, "o",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_pipe_pane_exec,
@@ -46,16 +50,21 @@ int
 cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_target_data	*data = self->data;
+	struct client		*c;
 	struct window_pane	*wp;
+	char			*command;
 	int			 old_fd, pipe_fd[2], null_fd, mode;
 
+	if ((c = cmd_find_client(ctx, NULL)) == NULL)
+		return (-1);
+
 	if (cmd_find_pane(ctx, data->target, NULL, &wp) == NULL)
 		return (-1);
 
 	/* Destroy the old pipe. */
 	old_fd = wp->pipe_fd;
 	if (wp->pipe_fd != -1) {
-		buffer_destroy(wp->pipe_buf);
+		bufferevent_free(wp->pipe_event);
 		close(wp->pipe_fd);
 		wp->pipe_fd = -1;
 	}
@@ -70,12 +79,12 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	 *
 	 *	bind ^p pipep -o 'cat >>~/output'
 	 */
-	if (data->chflags & CMD_CHFLAG('o') && old_fd != -1)
+	if (cmd_check_flag(data->chflags, 'o') && old_fd != -1)
 		return (0);
 
 	/* Open the new pipe. */
-	if (pipe(pipe_fd) != 0) {
-		ctx->error(ctx, "pipe error: %s", strerror(errno));
+	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, pipe_fd) != 0) {
+		ctx->error(ctx, "socketpair error: %s", strerror(errno));
 		return (-1);
 	}
 
@@ -83,11 +92,11 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	switch (fork()) {
 	case -1:
 		ctx->error(ctx, "fork error: %s", strerror(errno));
-		return (-1);		
+		return (-1);
 	case 0:
 		/* Child process. */
 		close(pipe_fd[0]);
-		sigreset();
+		clear_signals();
 
 		if (dup2(pipe_fd[1], STDIN_FILENO) == -1)
 			_exit(1);
@@ -102,24 +111,38 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 		if (null_fd != STDOUT_FILENO && null_fd != STDERR_FILENO)
 			close(null_fd);
 
-		execl(_PATH_BSHELL, "sh", "-c", data->arg, (char *) NULL);
+		command = status_replace(c, NULL, data->arg, time(NULL), 0);
+		execl(_PATH_BSHELL, "sh", "-c", command, (char *) NULL);
 		_exit(1);
 	default:
 		/* Parent process. */
 		close(pipe_fd[1]);
 
 		wp->pipe_fd = pipe_fd[0];
-		wp->pipe_buf = buffer_create(BUFSIZ);
-		wp->pipe_off = BUFFER_USED(wp->in);
-		
+		wp->pipe_off = EVBUFFER_LENGTH(wp->event->input);
+
+		wp->pipe_event = bufferevent_new(wp->pipe_fd,
+		    NULL, NULL, cmd_pipe_pane_error_callback, wp);
+		bufferevent_enable(wp->pipe_event, EV_WRITE);
+
 		if ((mode = fcntl(wp->pipe_fd, F_GETFL)) == -1)
 			fatal("fcntl failed");
 		if (fcntl(wp->pipe_fd, F_SETFL, mode|O_NONBLOCK) == -1)
 			fatal("fcntl failed");
 		if (fcntl(wp->pipe_fd, F_SETFD, FD_CLOEXEC) == -1)
-			fatal("fcntl failed");	
+			fatal("fcntl failed");
 		return (0);
 	}
+}
 
-	return (0);
+/* ARGSUSED */
+void
+cmd_pipe_pane_error_callback(
+    unused struct bufferevent *bufev, unused short what, void *data)
+{
+	struct window_pane	*wp = data;
+
+	bufferevent_free(wp->pipe_event);
+	close(wp->pipe_fd);
+	wp->pipe_fd = -1;
 }
