@@ -1,4 +1,4 @@
-/* $Id: cmd-respawn-window.c,v 1.22 2009/09/16 12:36:27 nicm Exp $ */
+/* $Id: cmd-respawn-window.c,v 1.25 2009/12/04 22:14:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,7 +31,7 @@ int	cmd_respawn_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_respawn_window_entry = {
 	"respawn-window", "respawnw",
 	"[-k] " CMD_TARGET_WINDOW_USAGE " [command]",
-	CMD_ARG01, CMD_CHFLAG('k'),
+	CMD_ARG01, "k",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_respawn_window_exec,
@@ -54,7 +54,7 @@ cmd_respawn_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return (-1);
 	w = wl->window;
 
-	if (!(data->chflags & CMD_CHFLAG('k'))) {
+	if (!cmd_check_flag(data->chflags, 'k')) {
 		TAILQ_FOREACH(wp, &w->panes, entry) {
 			if (wp->fd == -1)
 				continue;
@@ -72,7 +72,7 @@ cmd_respawn_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	wp = TAILQ_FIRST(&w->panes);
 	TAILQ_REMOVE(&w->panes, wp, entry);
 	layout_free(w);
- 	window_destroy_panes(w);
+	window_destroy_panes(w);
 	TAILQ_INSERT_HEAD(&w->panes, wp, entry);
 	window_pane_resize(wp, w->sx, w->sy);
 	if (window_pane_spawn(
@@ -80,6 +80,7 @@ cmd_respawn_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 		ctx->error(ctx, "respawn window failed: %s", cause);
 		xfree(cause);
 		environ_free(&env);
+		server_destroy_pane(wp);
 		return (-1);
 	}
 	layout_init(w);
