@@ -1,4 +1,4 @@
-/* $Id: cmd-save-buffer.c,v 1.9 2009/10/28 23:08:52 tcunha Exp $ */
+/* $Id: cmd-save-buffer.c,v 1.11 2010/07/02 02:52:13 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Tiago Cunha <me@tiagocunha.org>
@@ -33,7 +33,7 @@ int	cmd_save_buffer_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_save_buffer_entry = {
 	"save-buffer", "saveb",
 	"[-a] " CMD_BUFFER_SESSION_USAGE " path",
-	CMD_ARG1, CMD_CHFLAG('a'),
+	CMD_ARG1, "a",
 	cmd_buffer_init,
 	cmd_buffer_parse,
 	cmd_save_buffer_exec,
@@ -48,7 +48,7 @@ cmd_save_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session		*s;
 	struct paste_buffer	*pb;
 	mode_t			mask;
-	FILE			*f;
+	FILE			*f, *close_f;
 
 	if ((s = cmd_find_session(ctx, data->target)) == NULL)
 		return (-1);
@@ -65,15 +65,25 @@ cmd_save_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		}
 	}
 
-	mask = umask(S_IRWXG | S_IRWXO);
-	if (data->chflags & CMD_CHFLAG('a'))
-		f = fopen(data->arg, "ab");
-	else
-		f = fopen(data->arg, "wb");
-	umask(mask);
-	if (f == NULL) {
-		ctx->error(ctx, "%s: %s", data->arg, strerror(errno));
-		return (-1);
+	if (strcmp(data->arg, "-") == 0) {
+		if (ctx->cmdclient == NULL) {
+			ctx->error(ctx, "%s: can't write to stdout", data->arg);
+			return (-1);
+		}
+		f = ctx->cmdclient->stdout_file;
+		close_f = NULL;
+	} else {
+		mask = umask(S_IRWXG | S_IRWXO);
+		if (cmd_check_flag(data->chflags, 'a'))
+			f = fopen(data->arg, "ab");
+		else
+			f = fopen(data->arg, "wb");
+		umask(mask);
+		if (f == NULL) {
+			ctx->error(ctx, "%s: %s", data->arg, strerror(errno));
+			return (-1);
+		}
+		close_f = f;
 	}
 
 	if (fwrite(pb->data, 1, pb->size, f) != pb->size) {
@@ -82,7 +92,8 @@ cmd_save_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 	    	return (-1);
 	}
 
-	fclose(f);
+	if (close_f != NULL)
+		fclose(close_f);
 
 	return (0);
 }
