@@ -1,4 +1,4 @@
-/* $Id: cmd-send-keys.c,v 1.22 2009/09/22 14:03:11 tcunha Exp $ */
+/* $Id: cmd-send-keys.c,v 1.25 2010/05/22 21:56:04 micahcowan Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -33,14 +33,14 @@ size_t	cmd_send_keys_print(struct cmd *, char *, size_t);
 
 struct cmd_send_keys_data {
 	char	*target;
-  	u_int	 nkeys;
+	u_int	 nkeys;
 	int	*keys;
 };
 
 const struct cmd_entry cmd_send_keys_entry = {
 	"send-keys", "send",
 	"[-t target-pane] key ...",
-	0, 0,
+	0, "",
 	NULL,
 	cmd_send_keys_parse,
 	cmd_send_keys_exec,
@@ -105,16 +105,17 @@ cmd_send_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
 	struct cmd_send_keys_data	*data = self->data;
 	struct window_pane		*wp;
+	struct session			*s;
 	u_int				 i;
 
 	if (data == NULL)
 		return (-1);
 
-	if (cmd_find_pane(ctx, data->target, NULL, &wp) == NULL)
+	if (cmd_find_pane(ctx, data->target, &s, &wp) == NULL)
 		return (-1);
 
 	for (i = 0; i < data->nkeys; i++)
-		window_pane_key(wp, ctx->curclient, data->keys[i]);
+		window_pane_key(wp, s, data->keys[i]);
 
 	return (0);
 }
