@@ -1,4 +1,4 @@
-/* $Id: cmd-split-window.c,v 1.28 2009/09/22 14:06:40 tcunha Exp $ */
+/* $Id: cmd-split-window.c,v 1.35 2010/07/02 02:49:19 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -44,8 +44,8 @@ struct cmd_split_window_data {
 
 const struct cmd_entry cmd_split_window_entry = {
 	"split-window", "splitw",
-	"[-dhv] [-p percentage|-l size] [-t target-window] [command]",
-	0, 0,
+	"[-dhv] [-p percentage|-l size] [-t target-pane] [command]",
+	0, "",
 	cmd_split_window_init,
 	cmd_split_window_parse,
 	cmd_split_window_exec,
@@ -69,7 +69,7 @@ cmd_split_window_init(struct cmd *self, int key)
 	switch (key) {
 	case '%':
 		data->flag_horizontal = 1;
-		break;		
+		break;
 	case '"':
 		data->flag_horizontal = 0;
 		break;
@@ -148,15 +148,16 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	struct session			*s;
 	struct winlink			*wl;
 	struct window			*w;
-	struct window_pane		*wp;
+	struct window_pane		*wp, *new_wp = NULL;
 	struct environ			 env;
 	char		 		*cmd, *cwd, *cause;
 	const char			*shell;
 	u_int				 hlimit;
 	int				 size;
 	enum layout_type		 type;
+	struct layout_cell		*lc;
 
-	if ((wl = cmd_find_window(ctx, data->target, &s)) == NULL)
+	if ((wl = cmd_find_pane(ctx, data->target, &s, &wp)) == NULL)
 		return (-1);
 	w = wl->window;
 
@@ -168,38 +169,47 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	cmd = data->cmd;
 	if (cmd == NULL)
 		cmd = options_get_string(&s->options, "default-command");
-	if (ctx->cmdclient == NULL || ctx->cmdclient->cwd == NULL)
-		cwd = options_get_string(&s->options, "default-path");
-	else
-		cwd = ctx->cmdclient->cwd;
-
-	size = -1;
-	if (data->size != -1)
-		size = data->size;
-	else if (data->percentage != -1)
-		size = (w->active->sy * data->percentage) / 100;
-	hlimit = options_get_number(&s->options, "history-limit");
+	cwd = options_get_string(&s->options, "default-path");
+	if (*cwd == '\0') {
+		if (ctx->cmdclient != NULL && ctx->cmdclient->cwd != NULL)
+			cwd = ctx->cmdclient->cwd;
+		else
+			cwd = s->cwd;
+	}
 
 	type = LAYOUT_TOPBOTTOM;
 	if (data->flag_horizontal)
 		type = LAYOUT_LEFTRIGHT;
 
+	size = -1;
+	if (data->size != -1)
+		size = data->size;
+	else if (data->percentage != -1) {
+		if (type == LAYOUT_TOPBOTTOM)
+			size = (wp->sy * data->percentage) / 100;
+		else
+			size = (wp->sx * data->percentage) / 100;
+	}
+	hlimit = options_get_number(&s->options, "history-limit");
+
 	shell = options_get_string(&s->options, "default-shell");
 	if (*shell == '\0' || areshell(shell))
 		shell = _PATH_BSHELL;
 
-	wp = window_add_pane(w, hlimit);
-	if (window_pane_spawn(wp, cmd, shell, cwd, &env, s->tio, &cause) != 0)
-		goto error;
-	if (layout_split_pane(w->active, type, size, wp) != 0) {
+	if ((lc = layout_split_pane(wp, type, size)) == NULL) {
 		cause = xstrdup("pane too small");
 		goto error;
 	}
+	new_wp = window_add_pane(w, hlimit);
+	if (window_pane_spawn(
+	    new_wp, cmd, shell, cwd, &env, s->tio, &cause) != 0)
+		goto error;
+	layout_assign_pane(lc, new_wp);
 
 	server_redraw_window(w);
 
 	if (!data->flag_detached) {
-		window_set_active_pane(w, wp);
+		window_set_active_pane(w, new_wp);
 		session_select(s, wl->idx);
 		server_redraw_session(s);
 	} else
@@ -210,8 +220,8 @@ cmd_split_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 
 error:
 	environ_free(&env);
-	if (wp != NULL)
-		window_remove_pane(w, wp);
+	if (new_wp != NULL)
+		window_remove_pane(w, new_wp);
 	ctx->error(ctx, "create pane failed: %s", cause);
 	xfree(cause);
 	return (-1);
