@@ -1,4 +1,4 @@
-/* $Id: cmd-suspend-client.c,v 1.4 2009/07/28 22:12:16 tcunha Exp $ */
+/* $Id: cmd-suspend-client.c,v 1.5 2009/11/14 17:56:39 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -37,7 +37,7 @@ struct cmd_suspend_client_data {
 const struct cmd_entry cmd_suspend_client_entry = {
 	"suspend-client", "suspendc",
 	"[-c target-client]",
-	0, 0,
+	0, "",
 	cmd_target_init,
 	cmd_target_parse,
 	cmd_suspend_client_exec,
