@@ -1,4 +1,4 @@
-/* $Id: cmd-swap-window.c,v 1.18 2009/10/11 23:38:16 tcunha Exp $ */
+/* $Id: cmd-swap-window.c,v 1.19 2009/11/14 17:56:39 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,7 +31,7 @@ int	cmd_swap_window_exec(struct cmd *, struct cmd_ctx *);
 const struct cmd_entry cmd_swap_window_entry = {
 	"swap-window", "swapw",
 	"[-d] " CMD_SRCDST_WINDOW_USAGE,
-	0, CMD_CHFLAG('d'),
+	0, "d",
 	cmd_srcdst_init,
 	cmd_srcdst_parse,
 	cmd_swap_window_exec,
@@ -68,7 +68,7 @@ cmd_swap_window_exec(struct cmd *self, struct cmd_ctx *ctx)
 	wl_dst->window = wl_src->window;
 	wl_src->window = w;
 
-	if (!(data->chflags & CMD_CHFLAG('d'))) {
+	if (!cmd_check_flag(data->chflags, 'd')) {
 		session_select(dst, wl_dst->idx);
 		if (src != dst)
 			session_select(src, wl_src->idx);
