@@ -1,4 +1,4 @@
-/* $Id: grid.c,v 1.34 2009/10/15 01:55:12 tcunha Exp $ */
+/* $Id: grid.c,v 1.37 2010/04/05 05:11:43 micahcowan Exp $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -46,18 +46,9 @@ const struct grid_cell grid_default_cell = { 0, 0, 8, 8, ' ' };
 	    gc, sizeof gd->linedata[py].utf8data[px]);		\
 } while (0)
 
-int	grid_check_x(struct grid *, u_int);
 int	grid_check_y(struct grid *, u_int);
 
 #ifdef DEBUG
-int
-grid_check_x(struct grid *gd, u_int px)
-{
-	if ((px) >= (gd)->sx)
-		log_fatalx("x out of range: %u", px);
-	return (0);
-}
-
 int
 grid_check_y(struct grid *gd, u_int py)
 {
@@ -66,16 +57,6 @@ grid_check_y(struct grid *gd, u_int py)
 	return (0);
 }
 #else
-int
-grid_check_x(struct grid *gd, u_int px)
-{
-	if ((px) >= (gd)->sx) {
-		log_debug("x out of range: %u", px);
-		return (-1);
-	}
-	return (0);
-}
-
 int
 grid_check_y(struct grid *gd, u_int py)
 {
@@ -170,7 +151,7 @@ grid_collect_history(struct grid *gd)
 {
 	u_int	yy;
 
- 	GRID_DEBUG(gd, "");
+	GRID_DEBUG(gd, "");
 
 	if (gd->hsize < gd->hlimit)
 		return;
@@ -183,7 +164,7 @@ grid_collect_history(struct grid *gd)
 	gd->hsize -= yy;
 }
 
-/* 
+/*
  * Scroll the entire visible screen, moving one line into the history. Just
  * allocate a new line at the bottom and move the history size indicator.
  */
@@ -192,12 +173,12 @@ grid_scroll_history(struct grid *gd)
 {
 	u_int	yy;
 
- 	GRID_DEBUG(gd, "");
+	GRID_DEBUG(gd, "");
 
 	yy = gd->hsize + gd->sy;
 	gd->linedata = xrealloc(gd->linedata, yy + 1, sizeof *gd->linedata);
 	memset(&gd->linedata[yy], 0, sizeof gd->linedata[yy]);
-	
+
 	gd->hsize++;
 }
 
@@ -208,7 +189,7 @@ grid_scroll_history_region(struct grid *gd, u_int upper, u_int lower)
 	struct grid_line	*gl_history, *gl_upper, *gl_lower;
 	u_int			 yy;
 
- 	GRID_DEBUG(gd, "upper=%u, lower=%u", upper, lower);
+	GRID_DEBUG(gd, "upper=%u, lower=%u", upper, lower);
 
 	/* Create a space for a new line. */
 	yy = gd->hsize + gd->sy;
@@ -270,8 +251,6 @@ grid_expand_line_utf8(struct grid *gd, u_int py, u_int sx)
 const struct grid_cell *
 grid_peek_cell(struct grid *gd, u_int px, u_int py)
 {
-	if (grid_check_x(gd, px) != 0)
-		return (&grid_default_cell);
 	if (grid_check_y(gd, py) != 0)
 		return (&grid_default_cell);
 
@@ -284,8 +263,6 @@ grid_peek_cell(struct grid *gd, u_int px, u_int py)
 struct grid_cell *
 grid_get_cell(struct grid *gd, u_int px, u_int py)
 {
-	if (grid_check_x(gd, px) != 0)
-		return (NULL);
 	if (grid_check_y(gd, py) != 0)
 		return (NULL);
 
@@ -298,8 +275,6 @@ void
 grid_set_cell(
     struct grid *gd, u_int px, u_int py, const struct grid_cell *gc)
 {
-	if (grid_check_x(gd, px) != 0)
-		return;
 	if (grid_check_y(gd, py) != 0)
 		return;
 
@@ -311,8 +286,6 @@ grid_set_cell(
 const struct grid_utf8 *
 grid_peek_utf8(struct grid *gd, u_int px, u_int py)
 {
-	if (grid_check_x(gd, px) != 0)
-		return (NULL);
 	if (grid_check_y(gd, py) != 0)
 		return (NULL);
 
@@ -325,8 +298,6 @@ grid_peek_utf8(struct grid *gd, u_int px, u_int py)
 struct grid_utf8 *
 grid_get_utf8(struct grid *gd, u_int px, u_int py)
 {
-	if (grid_check_x(gd, px) != 0)
-		return (NULL);
 	if (grid_check_y(gd, py) != 0)
 		return (NULL);
 
@@ -339,8 +310,6 @@ void
 grid_set_utf8(
     struct grid *gd, u_int px, u_int py, const struct grid_utf8 *gc)
 {
-	if (grid_check_x(gd, px) != 0)
-		return;
 	if (grid_check_y(gd, py) != 0)
 		return;
 
@@ -354,7 +323,7 @@ grid_clear(struct grid *gd, u_int px, u_int py, u_int nx, u_int ny)
 {
 	u_int	xx, yy;
 
- 	GRID_DEBUG(gd, "px=%u, py=%u, nx=%u, ny=%u", px, py, nx, ny);
+	GRID_DEBUG(gd, "px=%u, py=%u, nx=%u, ny=%u", px, py, nx, ny);
 
 	if (nx == 0 || ny == 0)
 		return;
@@ -364,10 +333,6 @@ grid_clear(struct grid *gd, u_int px, u_int py, u_int nx, u_int ny)
 		return;
 	}
 
-	if (grid_check_x(gd, px) != 0)
-		return;
-	if (grid_check_x(gd, px + nx - 1) != 0)
-		return;
 	if (grid_check_y(gd, py) != 0)
 		return;
 	if (grid_check_y(gd, py + ny - 1) != 0)
@@ -395,7 +360,7 @@ grid_clear_lines(struct grid *gd, u_int py, u_int ny)
 	struct grid_line	*gl;
 	u_int			 yy;
 
- 	GRID_DEBUG(gd, "py=%u, ny=%u", py, ny);
+	GRID_DEBUG(gd, "py=%u, ny=%u", py, ny);
 
 	if (ny == 0)
 		return;
@@ -421,7 +386,7 @@ grid_move_lines(struct grid *gd, u_int dy, u_int py, u_int ny)
 {
 	u_int	yy;
 
- 	GRID_DEBUG(gd, "dy=%u, py=%u, ny=%u", dy, py, ny);
+	GRID_DEBUG(gd, "dy=%u, py=%u, ny=%u", dy, py, ny);
 
 	if (ny == 0 || py == dy)
 		return;
@@ -460,17 +425,11 @@ grid_move_cells(struct grid *gd, u_int dx, u_int px, u_int py, u_int nx)
 	struct grid_line	*gl;
 	u_int			 xx;
 
- 	GRID_DEBUG(gd, "dx=%u, px=%u, py=%u, nx=%u", dx, px, py, nx);
+	GRID_DEBUG(gd, "dx=%u, px=%u, py=%u, nx=%u", dx, px, py, nx);
 
 	if (nx == 0 || px == dx)
 		return;
 
-	if (grid_check_x(gd, px) != 0)
-		return;
-	if (grid_check_x(gd, px + nx - 1) != 0)
-		return;
-	if (grid_check_x(gd, dx + nx - 1) != 0)
-		return;
 	if (grid_check_y(gd, py) != 0)
 		return;
 	gl = &gd->linedata[py];
@@ -499,11 +458,11 @@ grid_move_cells(struct grid *gd, u_int dx, u_int px, u_int py, u_int nx)
 char *
 grid_string_cells(struct grid *gd, u_int px, u_int py, u_int nx)
 {
- 	const struct grid_cell	*gc;
- 	const struct grid_utf8	*gu;
+	const struct grid_cell	*gc;
+	const struct grid_utf8	*gu;
 	char			*buf;
-	size_t			 len, off;
-	u_int			 xx, i;
+	size_t			 len, off, size;
+	u_int			 xx;
 
 	GRID_DEBUG(gd, "px=%u, py=%u, nx=%u", px, py, nx);
 
@@ -517,17 +476,15 @@ grid_string_cells(struct grid *gd, u_int px, u_int py, u_int nx)
 			continue;
 
 		if (gc->flags & GRID_FLAG_UTF8) {
-			while (len < off + UTF8_SIZE + 1) {
+			gu = grid_peek_utf8(gd, xx, py);
+
+			size = grid_utf8_size(gu);
+			while (len < off + size + 1) {
 				buf = xrealloc(buf, 2, len);
 				len *= 2;
 			}
 
-			gu = grid_peek_utf8(gd, xx, py);
-			for (i = 0; i < UTF8_SIZE; i++) {
-				if (gu->data[i] == 0xff)
-					break;
-				buf[off++] = gu->data[i];
-			}
+			off += grid_utf8_copy(gu, buf + off, len - off);
 		} else {
 			while (len < off + 2) {
 				buf = xrealloc(buf, 2, len);
@@ -537,14 +494,14 @@ grid_string_cells(struct grid *gd, u_int px, u_int py, u_int nx)
 			buf[off++] = gc->data;
 		}
 	}
-	
+
 	while (off > 0 && buf[off - 1] == ' ')
 		off--;
 	buf[off] = '\0';
 	return (buf);
 }
 
-/* 
+/*
  * Duplicate a set of lines between two grids. If there aren't enough lines in
  * either source or destination, the number of lines is limited to the number
  * available.
