@@ -1,4 +1,4 @@
-/* $Id: input-keys.c,v 1.39 2009/10/28 23:05:01 tcunha Exp $ */
+/* $Id: input-keys.c,v 1.44 2009/12/04 22:14:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -50,49 +50,27 @@ struct input_key_ent input_keys[] = {
 	{ KEYC_F3,		"\033OR",	0 },
 	{ KEYC_F4,		"\033OS",	0 },
 	{ KEYC_F5,		"\033[15~",	0 },
-	{ KEYC_F5|KEYC_CTRL,	"\033[15^",	0 },
 	{ KEYC_F6,		"\033[17~",	0 },
-	{ KEYC_F6|KEYC_CTRL,	"\033[17^",	0 },
 	{ KEYC_F7,		"\033[18~",	0 },
-	{ KEYC_F7|KEYC_CTRL,	"\033[18^",	0 },
 	{ KEYC_F8,		"\033[19~",	0 },
-	{ KEYC_F8|KEYC_CTRL,	"\033[19^",	0 },
 	{ KEYC_F9,		"\033[20~",	0 },
-	{ KEYC_F9|KEYC_CTRL,	"\033[20^",	0 },
 	{ KEYC_F10,		"\033[21~",	0 },
-	{ KEYC_F10|KEYC_CTRL,	"\033[21^",	0 },
 	{ KEYC_F11,		"\033[23~",	0 },
-	{ KEYC_F1|KEYC_CTRL,	"\033[23^",	0 },
 	{ KEYC_F12,		"\033[24~",	0 },
-	{ KEYC_F12|KEYC_CTRL,	"\033[24^",	0 },
 	{ KEYC_F13,		"\033[25~",	0 },
-	{ KEYC_F13|KEYC_CTRL,	"\033[25^",	0 },
 	{ KEYC_F14,		"\033[26~",	0 },
-	{ KEYC_F14|KEYC_CTRL,	"\033[26^",	0 },
 	{ KEYC_F15,		"\033[28~",	0 },
-	{ KEYC_F15|KEYC_CTRL,	"\033[28^",	0 },
 	{ KEYC_F16,		"\033[29~",	0 },
-	{ KEYC_F16|KEYC_CTRL,	"\033[29^",	0 },
 	{ KEYC_F17,		"\033[31~",	0 },
-	{ KEYC_F17|KEYC_CTRL,	"\033[31^",	0 },
 	{ KEYC_F18,		"\033[32~",	0 },
-	{ KEYC_F18|KEYC_CTRL,	"\033[32^",	0 },
 	{ KEYC_F19,		"\033[33~",	0 },
-	{ KEYC_F19|KEYC_CTRL,	"\033[33^",	0 },
 	{ KEYC_F20,		"\033[34~",	0 },
-	{ KEYC_F20|KEYC_CTRL,	"\033[34^",	0 },
 	{ KEYC_IC,		"\033[2~",	0 },
-	{ KEYC_IC|KEYC_CTRL,	"\033[2^",	0 },
 	{ KEYC_DC,		"\033[3~",	0 },
-	{ KEYC_DC|KEYC_CTRL,	"\033[3^",	0 },
 	{ KEYC_HOME,		"\033[1~",	0 },
-	{ KEYC_HOME|KEYC_CTRL,	"\033[1^",	0 },
 	{ KEYC_END,		"\033[4~",	0 },
-	{ KEYC_END|KEYC_CTRL,	"\033[4^",	0 },
 	{ KEYC_NPAGE,		"\033[6~",	0 },
-	{ KEYC_NPAGE|KEYC_CTRL,	"\033[6^",	0 },
 	{ KEYC_PPAGE,		"\033[5~",	0 },
-	{ KEYC_PPAGE|KEYC_CTRL,	"\033[5^",	0 },
 	{ KEYC_BTAB,		"\033[Z",	0 },
 
 	/*
@@ -103,7 +81,7 @@ struct input_key_ent input_keys[] = {
 	{ KEYC_DOWN|KEYC_CTRL,	"\033[B",	INPUTKEY_CURSOR },
 	{ KEYC_RIGHT|KEYC_CTRL,	"\033[C",	INPUTKEY_CURSOR },
 	{ KEYC_LEFT|KEYC_CTRL,	"\033[D",	INPUTKEY_CURSOR },
-	
+
 	{ KEYC_UP,		"\033OA",	INPUTKEY_CURSOR },
 	{ KEYC_DOWN,		"\033OB",	INPUTKEY_CURSOR },
 	{ KEYC_RIGHT,		"\033OC",	INPUTKEY_CURSOR },
@@ -120,39 +98,39 @@ struct input_key_ent input_keys[] = {
 	{ KEYC_LEFT,		"\033[D",	0 },
 
 	/* Keypad keys. Keypad versions must come first. */
-	{ KEYC_KP_SLASH,	"/",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_STAR,		"*",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_MINUS,	"-",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_SEVEN,	"7",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_EIGHT,	"8",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_NINE,		"9",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_PLUS,		"+",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_FOUR,		"4",		INPUTKEY_KEYPAD },	
-	{ KEYC_KP_FIVE,		"5",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_SIX,		"6",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_ONE,		"1",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_TWO,		"2",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_THREE,	"3",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_ENTER,	"\n",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_ZERO,		"0",		INPUTKEY_KEYPAD },
-	{ KEYC_KP_PERIOD,	".",		INPUTKEY_KEYPAD },
+	{ KEYC_KP_SLASH,	"\033Oo",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_STAR,		"\033Oj",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_MINUS,	"\033Om",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_SEVEN,	"\033Ow",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_EIGHT,	"\033Ox",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_NINE,		"\033Oy",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_PLUS,		"\033Ok",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_FOUR,		"\033Ot",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_FIVE,		"\033Ou",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_SIX,		"\033Ov",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_ONE,		"\033Oq",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_TWO,		"\033Or",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_THREE,	"\033Os",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_ENTER,	"\033OM",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_ZERO,		"\033Op",	INPUTKEY_KEYPAD },
+	{ KEYC_KP_PERIOD,	"\033On",	INPUTKEY_KEYPAD },
 
-	{ KEYC_KP_SLASH,	"\033Oo",	0 },
-	{ KEYC_KP_STAR,		"\033Oj",	0 },
-	{ KEYC_KP_MINUS,	"\033Om",	0 },
-	{ KEYC_KP_SEVEN,	"\033Ow",	0 },
-	{ KEYC_KP_EIGHT,	"\033Ox",	0 },
-	{ KEYC_KP_NINE,		"\033Oy",	0 },
-	{ KEYC_KP_PLUS,		"\033Ok",	0 },
-	{ KEYC_KP_FOUR,		"\033Ot",	0 },
-	{ KEYC_KP_FIVE,		"\033Ou",	0 },
-	{ KEYC_KP_SIX,		"\033Ov",	0 },
-	{ KEYC_KP_ONE,		"\033Oq",	0 },
-	{ KEYC_KP_TWO,		"\033Or",	0 },
-	{ KEYC_KP_THREE,	"\033Os",	0 },
-	{ KEYC_KP_ENTER,	"\033OM",	0 },
-	{ KEYC_KP_ZERO,		"\033Op",	0 },
-	{ KEYC_KP_PERIOD,	"\033On",	0 },
+	{ KEYC_KP_SLASH,	"/",		0 },
+	{ KEYC_KP_STAR,		"*",		0 },
+	{ KEYC_KP_MINUS,	"-",		0 },
+	{ KEYC_KP_SEVEN,	"7",		0 },
+	{ KEYC_KP_EIGHT,	"8",		0 },
+	{ KEYC_KP_NINE,		"9",		0 },
+	{ KEYC_KP_PLUS,		"+",		0 },
+	{ KEYC_KP_FOUR,		"4",		0 },
+	{ KEYC_KP_FIVE,		"5",		0 },
+	{ KEYC_KP_SIX,		"6",		0 },
+	{ KEYC_KP_ONE,		"1",		0 },
+	{ KEYC_KP_TWO,		"2",		0 },
+	{ KEYC_KP_THREE,	"3",		0 },
+	{ KEYC_KP_ENTER,	"\n",		0 },
+	{ KEYC_KP_ZERO,		"0",		0 },
+	{ KEYC_KP_PERIOD,	".",		0 },
 };
 
 /* Translate a key code into an output key sequence. */
@@ -163,6 +141,7 @@ input_key(struct window_pane *wp, int key)
 	u_int			i;
 	size_t			dlen;
 	char		       *out;
+	u_char			ch;
 
 	log_debug2("writing key 0x%x", key);
 
@@ -172,18 +151,19 @@ input_key(struct window_pane *wp, int key)
 	 */
 	if (key != KEYC_NONE && (key & ~KEYC_ESCAPE) < 0x100) {
 		if (key & KEYC_ESCAPE)
-			buffer_write8(wp->out, '\033');
-		buffer_write8(wp->out, (uint8_t) (key & ~KEYC_ESCAPE));
+			bufferevent_write(wp->event, "\033", 1);
+		ch = key & ~KEYC_ESCAPE;
+		bufferevent_write(wp->event, &ch, 1);
 		return;
 	}
 
-	/* 
+	/*
 	 * Then try to look this up as an xterm key, if the flag to output them
 	 * is set.
 	 */
 	if (options_get_number(&wp->window->options, "xterm-keys")) {
 		if ((out = xterm_keys_lookup(key)) != NULL) {
-			buffer_write(wp->out, out, strlen(out));
+			bufferevent_write(wp->event, out, strlen(out));
 			xfree(out);
 			return;
 		}
@@ -214,18 +194,19 @@ input_key(struct window_pane *wp, int key)
 
 	/* Prefix a \033 for escape. */
 	if (key & KEYC_ESCAPE)
-		buffer_write8(wp->out, '\033');
-	buffer_write(wp->out, ike->data, dlen);
+		bufferevent_write(wp->event, "\033", 1);
+	bufferevent_write(wp->event, ike->data, dlen);
 }
 
 /* Translate mouse and output. */
 void
 input_mouse(struct window_pane *wp, struct mouse_event *m)
 {
+	char	out[8];
+
 	if (wp->screen->mode & MODE_MOUSE) {
-		buffer_write(wp->out, "\033[M", 3);
-		buffer_write8(wp->out, m->b + 32);
-		buffer_write8(wp->out, m->x + 33);
-		buffer_write8(wp->out, m->y + 33);
+		xsnprintf(out, sizeof out,
+		    "\033[M%c%c%c", m->b + 32, m->x + 33, m->y + 33);
+		bufferevent_write(wp->event, out, strlen(out));
 	}
 }
