@@ -1,4 +1,4 @@
-/* $Id: key-bindings.c,v 1.83 2009/10/06 14:14:07 tcunha Exp $ */
+/* $Id: key-bindings.c,v 1.94 2010/07/02 02:43:01 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -61,11 +61,11 @@ key_bindings_add(int key, int can_repeat, struct cmd_list *cmdlist)
 	struct key_binding	*bd;
 
 	key_bindings_remove(key);
-		    
+
 	bd = xmalloc(sizeof *bd);
 	bd->key = key;
 	SPLAY_INSERT(key_bindings, &key_bindings, bd);
-	
+
 	bd->can_repeat = can_repeat;
 	bd->cmdlist = cmdlist;
 }
@@ -104,9 +104,9 @@ key_bindings_init(void)
 	} table[] = {
 		{ ' ',			  0, &cmd_next_layout_entry },
 		{ '!', 			  0, &cmd_break_pane_entry },
-		{ '"', 			  0, &cmd_split_window_entry },	
-		{ '%', 			  0, &cmd_split_window_entry },	
+		{ '"', 			  0, &cmd_split_window_entry },
 		{ '#', 			  0, &cmd_list_buffers_entry },
+		{ '%', 			  0, &cmd_split_window_entry },
 		{ '&', 			  0, &cmd_confirm_before_entry },
 		{ ',', 			  0, &cmd_command_prompt_entry },
 		{ '-', 			  0, &cmd_delete_buffer_entry },
@@ -122,19 +122,22 @@ key_bindings_init(void)
 		{ '8', 			  0, &cmd_select_window_entry },
 		{ '9', 			  0, &cmd_select_window_entry },
 		{ ':', 			  0, &cmd_command_prompt_entry },
+		{ '=', 			  0, &cmd_choose_buffer_entry },
 		{ '?', 			  0, &cmd_list_keys_entry },
+		{ 'D',			  0, &cmd_choose_client_entry },
 		{ '[', 			  0, &cmd_copy_mode_entry },
-		{ '\'',			  0, &cmd_select_prompt_entry },
+		{ '\'',			  0, &cmd_command_prompt_entry },
+		{ '\002', /* C-b */	  0, &cmd_send_prefix_entry },
+		{ '\017', /* C-o */	  0, &cmd_rotate_window_entry },
 		{ '\032', /* C-z */	  0, &cmd_suspend_client_entry },
 		{ ']', 			  0, &cmd_paste_buffer_entry },
 		{ 'c', 			  0, &cmd_new_window_entry },
 		{ 'd', 			  0, &cmd_detach_client_entry },
-		{ 'D',			  0, &cmd_choose_client_entry },
 		{ 'f', 			  0, &cmd_command_prompt_entry },
 		{ 'i',			  0, &cmd_display_message_entry },
 		{ 'l', 			  0, &cmd_last_window_entry },
 		{ 'n', 			  0, &cmd_next_window_entry },
-		{ 'o', 			  0, &cmd_down_pane_entry },
+		{ 'o', 			  0, &cmd_select_pane_entry },
 		{ 'p', 			  0, &cmd_previous_window_entry },
 		{ 'q',			  0, &cmd_display_panes_entry },
 		{ 'r', 			  0, &cmd_refresh_client_entry },
@@ -144,26 +147,28 @@ key_bindings_init(void)
 		{ 'x', 			  0, &cmd_confirm_before_entry },
 		{ '{',			  0, &cmd_swap_pane_entry },
 		{ '}',			  0, &cmd_swap_pane_entry },
-		{ '\002', /* C-b */	  0, &cmd_send_prefix_entry },
+		{ '~',			  0, &cmd_show_messages_entry },
 		{ '1' | KEYC_ESCAPE,	  0, &cmd_select_layout_entry },
 		{ '2' | KEYC_ESCAPE,	  0, &cmd_select_layout_entry },
 		{ '3' | KEYC_ESCAPE,	  0, &cmd_select_layout_entry },
 		{ '4' | KEYC_ESCAPE,	  0, &cmd_select_layout_entry },
+		{ '5' | KEYC_ESCAPE,	  0, &cmd_select_layout_entry },
 		{ KEYC_PPAGE, 		  0, &cmd_copy_mode_entry },
 		{ 'n' | KEYC_ESCAPE, 	  0, &cmd_next_window_entry },
+		{ 'o' | KEYC_ESCAPE,	  0, &cmd_rotate_window_entry },
 		{ 'p' | KEYC_ESCAPE, 	  0, &cmd_previous_window_entry },
-		{ KEYC_UP, 		  0, &cmd_up_pane_entry },
-		{ KEYC_DOWN, 		  0, &cmd_down_pane_entry },
+		{ KEYC_UP, 		  1, &cmd_select_pane_entry },
+		{ KEYC_DOWN, 		  1, &cmd_select_pane_entry },
+		{ KEYC_LEFT, 		  1, &cmd_select_pane_entry },
+		{ KEYC_RIGHT, 		  1, &cmd_select_pane_entry },
 		{ KEYC_UP | KEYC_ESCAPE,  1, &cmd_resize_pane_entry },
 		{ KEYC_DOWN | KEYC_ESCAPE,  1, &cmd_resize_pane_entry },
 		{ KEYC_LEFT | KEYC_ESCAPE,  1, &cmd_resize_pane_entry },
 		{ KEYC_RIGHT | KEYC_ESCAPE, 1, &cmd_resize_pane_entry },
 		{ KEYC_UP | KEYC_CTRL,    1, &cmd_resize_pane_entry },
-		{ KEYC_DOWN | KEYC_CTRL,  1, &cmd_resize_pane_entry },	
+		{ KEYC_DOWN | KEYC_CTRL,  1, &cmd_resize_pane_entry },
 		{ KEYC_LEFT | KEYC_CTRL,  1, &cmd_resize_pane_entry },
 		{ KEYC_RIGHT | KEYC_CTRL, 1, &cmd_resize_pane_entry },
-		{ 'o' | KEYC_ESCAPE,	  0, &cmd_rotate_window_entry },
-		{ '\017', /* C-o */	  0, &cmd_rotate_window_entry },
 	};
 	u_int		 i;
 	struct cmd	*cmd;
@@ -173,34 +178,21 @@ key_bindings_init(void)
 
 	for (i = 0; i < nitems(table); i++) {
 		cmdlist = xmalloc(sizeof *cmdlist);
-		TAILQ_INIT(cmdlist);
+		TAILQ_INIT(&cmdlist->list);
+		cmdlist->references = 1;
 
 		cmd = xmalloc(sizeof *cmd);
 		cmd->entry = table[i].entry;
 		cmd->data = NULL;
 		if (cmd->entry->init != NULL)
 			cmd->entry->init(cmd, table[i].key);
-		TAILQ_INSERT_HEAD(cmdlist, cmd, qentry);
+		TAILQ_INSERT_HEAD(&cmdlist->list, cmd, qentry);
 
 		key_bindings_add(
 		    table[i].key | KEYC_PREFIX, table[i].can_repeat, cmdlist);
 	}
 }
 
-void
-key_bindings_free(void)
-{
-	struct key_binding	*bd;
-
-	key_bindings_clean();
-	while (!SPLAY_EMPTY(&key_bindings)) {
-		bd = SPLAY_ROOT(&key_bindings);
-		SPLAY_REMOVE(key_bindings, &key_bindings, bd);
-		cmd_list_free(bd->cmdlist);
-		xfree(bd);
-	}
-}
-
 void printflike2
 key_bindings_error(struct cmd_ctx *ctx, const char *fmt, ...)
 {
@@ -212,7 +204,7 @@ key_bindings_error(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_end(ap);
 
 	*msg = toupper((u_char) *msg);
- 	status_message_set(ctx->curclient, "%s", msg);
+	status_message_set(ctx->curclient, "%s", msg);
 	xfree(msg);
 }
 
@@ -222,12 +214,14 @@ key_bindings_print(struct cmd_ctx *ctx, const char *fmt, ...)
 	struct winlink	*wl = ctx->curclient->session->curw;
 	va_list		 ap;
 
-	if (wl->window->active->mode != &window_more_mode)
+	if (wl->window->active->mode != &window_copy_mode) {
 		window_pane_reset_mode(wl->window->active);
-	window_pane_set_mode(wl->window->active, &window_more_mode);
+		window_pane_set_mode(wl->window->active, &window_copy_mode);
+		window_copy_init_for_output(wl->window->active);
+	}
 
 	va_start(ap, fmt);
-	window_more_vadd(wl->window->active, fmt, ap);
+	window_copy_vadd(wl->window->active, fmt, ap);
 	va_end(ap);
 }
 
@@ -237,7 +231,7 @@ key_bindings_info(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_list	ap;
 	char   *msg;
 
-	if (be_quiet)
+	if (options_get_number(&global_options, "quiet"))
 		return;
 
 	va_start(ap, fmt);
@@ -245,14 +239,16 @@ key_bindings_info(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_end(ap);
 
 	*msg = toupper((u_char) *msg);
- 	status_message_set(ctx->curclient, "%s", msg);
+	status_message_set(ctx->curclient, "%s", msg);
 	xfree(msg);
 }
 
 void
 key_bindings_dispatch(struct key_binding *bd, struct client *c)
 {
-	struct cmd_ctx	 	 ctx;
+	struct cmd_ctx	 ctx;
+	struct cmd	*cmd;
+	int		 readonly;
 
 	ctx.msgdata = NULL;
 	ctx.curclient = c;
@@ -263,5 +259,15 @@ key_bindings_dispatch(struct key_binding *bd, struct client *c)
 
 	ctx.cmdclient = NULL;
 
+	readonly = 1;
+	TAILQ_FOREACH(cmd, &bd->cmdlist->list, qentry) {
+		if (!(cmd->entry->flags & CMD_READONLY))
+			readonly = 0;
+	}
+	if (!readonly && c->flags & CLIENT_READONLY) {
+		key_bindings_info(&ctx, "Client is read-only");
+		return;
+	}
+
 	cmd_list_exec(bd->cmdlist, &ctx);
 }
