@@ -1,4 +1,4 @@
-/* $Id: osdep-darwin.c,v 1.9 2009/02/13 00:43:04 nicm Exp $ */
+/* $Id: osdep-darwin.c,v 1.11 2009/05/04 17:58:27 nicm Exp $ */
 
 /*
  * Copyright (c) 2009 Joshua Elsasser <josh@elsasser.org>
@@ -16,8 +16,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef __APPLE__
-
 #include <sys/types.h>
 #include <sys/sysctl.h>
 
@@ -35,7 +33,7 @@ osdep_get_name(int fd, unused char *tty)
 	int	mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PID, 0 };
         size_t	size;
 	struct kinfo_proc kp;
-	
+
 	if ((mib[3] = tcgetpgrp(fd)) == -1)
 		return (NULL);
 
@@ -47,5 +45,3 @@ osdep_get_name(int fd, unused char *tty)
 
 	return (strdup(kp.kp_proc.p_comm));
 }
-
-#endif
