@@ -1,4 +1,4 @@
-/* $Id: osdep-freebsd.c,v 1.14 2009/02/13 00:43:04 nicm Exp $ */
+/* $Id: osdep-freebsd.c,v 1.19 2009/08/09 18:00:45 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -16,8 +16,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifdef __FreeBSD__
-
 #include <sys/param.h>
 #include <sys/proc.h>
 #include <sys/stat.h>
@@ -31,22 +29,58 @@
 #include <string.h>
 #include <unistd.h>
 
-char	*osdep_get_name(int, char *);
+struct kinfo_proc	*cmp_procs(struct kinfo_proc *, struct kinfo_proc *);
+char			*osdep_get_name(int, char *);
 
+#ifndef nitems
 #define nitems(_a) (sizeof((_a)) / sizeof((_a)[0]))
+#endif
 
 #define is_runnable(p) \
 	((p)->ki_stat == SRUN || (p)->ki_stat == SIDL)
 #define is_stopped(p) \
 	((p)->ki_stat == SSTOP || (p)->ki_stat == SZOMB)
 
+struct kinfo_proc *
+cmp_procs(struct kinfo_proc *p1, struct kinfo_proc *p2)
+{
+	if (is_runnable(p1) && !is_runnable(p2))
+		return (p1);
+	if (!is_runnable(p1) && is_runnable(p2))
+		return (p2);
+
+	if (is_stopped(p1) && !is_stopped(p2))
+		return (p1);
+	if (!is_stopped(p1) && is_stopped(p2))
+		return (p2);
+
+	if (p1->ki_estcpu > p2->ki_estcpu)
+		return (p1);
+	if (p1->ki_estcpu < p2->ki_estcpu)
+		return (p2);
+
+	if (p1->ki_slptime < p2->ki_slptime)
+		return (p1);
+	if (p1->ki_slptime > p2->ki_slptime)
+		return (p2);
+
+	if (strcmp(p1->ki_comm, p2->ki_comm) < 0)
+		return (p1);
+	if (strcmp(p1->ki_comm, p2->ki_comm) > 0)
+		return (p2);
+
+	if (p1->ki_pid > p2->ki_pid)
+		return (p1);
+	return (p2);
+}
+
 char *
 osdep_get_name(int fd, char *tty)
 {
 	int		 mib[4] = { CTL_KERN, KERN_PROC, KERN_PROC_PGRP, 0 };
 	struct stat	 sb;
 	size_t		 len;
-	struct kinfo_proc *buf, *newbuf, *p, *bestp;
+	struct kinfo_proc *buf, *newbuf, *bestp;
 	u_int		 i;
 	char		*name;
 
@@ -62,56 +96,24 @@ retry:
 		return (NULL);
 	len = (len * 5) / 4;
 
-	if ((newbuf = realloc(buf, len)) == NULL) {
-		free(buf);
-		return (NULL);
-	}
+	if ((newbuf = realloc(buf, len)) == NULL)
+		goto error;
 	buf = newbuf;
 
 	if (sysctl(mib, nitems(mib), buf, &len, NULL, 0) == -1) {
 		if (errno == ENOMEM)
 			goto retry;
-		free(buf);
-		return (NULL);
+		goto error;
 	}
 
 	bestp = NULL;
 	for (i = 0; i < len / sizeof (struct kinfo_proc); i++) {
 		if (buf[i].ki_tdev != sb.st_rdev)
 			continue;
-		p = &buf[i];
-		if (bestp == NULL) {
-			bestp = p;
-			continue;
-		}
-
-		if (is_runnable(p) && !is_runnable(bestp))
-			bestp = p;
-		else if (!is_runnable(p) && is_runnable(bestp))
-			continue;
-
-		if (!is_stopped(p) && is_stopped(bestp))
-			bestp = p;
-		else if (is_stopped(p) && !is_stopped(bestp))
-			continue;
-
-		if (p->ki_estcpu > bestp->ki_estcpu)
-			bestp = p;
-		else if (p->ki_estcpu < bestp->ki_estcpu)
-			continue;
-
-		if (p->ki_slptime < bestp->ki_slptime)
-			bestp = p;
-		else if (p->ki_slptime > bestp->ki_slptime)
-			continue;
-
-		if (strcmp(p->ki_comm, p->ki_comm) < 0)
-			bestp = p;
-		else if (strcmp(p->ki_comm, p->ki_comm) > 0)
-			continue;
-
-		if (p->ki_pid > bestp->ki_pid)
-			bestp = p;
+		if (bestp == NULL)
+			bestp = &buf[i];
+		else
+			bestp = cmp_procs(&buf[i], bestp);
 	}
 
 	name = NULL;
@@ -120,6 +122,8 @@ retry:
 
 	free(buf);
 	return (name);
-}
 
-#endif
+error:
+	free(buf);
+	return (NULL);
+}
