@@ -1,4 +1,4 @@
-/* $Id: screen-redraw.c,v 1.49 2009/10/28 23:17:28 tcunha Exp $ */
+/* $Id: screen-redraw.c,v 1.52 2010/02/05 01:31:06 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -22,6 +22,7 @@
 
 #include "tmux.h"
 
+int	screen_redraw_cell_border1(struct window_pane *, u_int, u_int);
 int	screen_redraw_cell_border(struct client *, u_int, u_int);
 int	screen_redraw_check_cell(struct client *, u_int, u_int);
 void	screen_redraw_draw_number(struct client *, struct window_pane *);
@@ -40,40 +41,49 @@ void	screen_redraw_draw_number(struct client *, struct window_pane *);
 #define CELL_JOIN 11
 #define CELL_OUTSIDE 12
 
+/* Check if cell is on the border of a particular pane. */
+int
+screen_redraw_cell_border1(struct window_pane *wp, u_int px, u_int py)
+{
+	/* Inside pane. */
+	if (px >= wp->xoff && px < wp->xoff + wp->sx &&
+	    py >= wp->yoff && py < wp->yoff + wp->sy)
+		return (0);
+
+	/* Left/right borders. */
+	if ((wp->yoff == 0 || py >= wp->yoff - 1) && py <= wp->yoff + wp->sy) {
+		if (wp->xoff != 0 && px == wp->xoff - 1)
+			return (1);
+		if (px == wp->xoff + wp->sx)
+			return (1);
+	}
+
+	/* Top/bottom borders. */
+	if ((wp->xoff == 0 || px >= wp->xoff - 1) && px <= wp->xoff + wp->sx) {
+		if (wp->yoff != 0 && py == wp->yoff - 1)
+			return (1);
+		if (py == wp->yoff + wp->sy)
+			return (1);
+	}
+
+	/* Outside pane. */
+	return (-1);
+}
+
 /* Check if a cell is on the pane border. */
 int
 screen_redraw_cell_border(struct client *c, u_int px, u_int py)
 {
 	struct window		*w = c->session->curw->window;
 	struct window_pane	*wp;
+	int			 retval;
 
 	/* Check all the panes. */
 	TAILQ_FOREACH(wp, &w->panes, entry) {
 		if (!window_pane_visible(wp))
 			continue;
-
-		/* Inside pane. */
-		if (px >= wp->xoff && px < wp->xoff + wp->sx &&
-		    py >= wp->yoff && py < wp->yoff + wp->sy)
-			return (0);
-
-		/* Left/right borders. */
-		if ((wp->yoff == 0 || py >= wp->yoff - 1) &&
-		    py <= wp->yoff + wp->sy) {
-			if (wp->xoff != 0 && px == wp->xoff - 1)
-				return (1);
-			if (px == wp->xoff + wp->sx)
-				return (1);
-		}
-
-		/* Top/bottom borders. */
-		if ((wp->xoff == 0 || px >= wp->xoff - 1) && 
-		    px <= wp->xoff + wp->sx) {
-			if (wp->yoff != 0 && py == wp->yoff - 1)
-				return (1);
-			if (py == wp->yoff + wp->sy)
-				return (1);
-		}
+		if ((retval = screen_redraw_cell_border1(wp, px, py)) != -1)
+			return (retval);
 	}
 
 	return (0);
@@ -105,7 +115,7 @@ screen_redraw_check_cell(struct client *c, u_int px, u_int py)
 		if (!screen_redraw_cell_border(c, px, py))
 			return (CELL_INSIDE);
 
-		/* 
+		/*
 		 * Construct a bitmask of whether the cells to the left (bit
 		 * 4), right, top, and bottom (bit 1) of this cell are borders.
 		 */
@@ -119,7 +129,7 @@ screen_redraw_check_cell(struct client *c, u_int px, u_int py)
 		if (py <= w->sy && screen_redraw_cell_border(c, px, py + 1))
 			borders |= 1;
 
-		/* 
+		/*
 		 * Figure out what kind of border this cell is. Only one bit
 		 * set doesn't make sense (can't have a border cell with no
 		 * others connected).
@@ -155,13 +165,14 @@ screen_redraw_check_cell(struct client *c, u_int px, u_int py)
 
 /* Redraw entire screen. */
 void
-screen_redraw_screen(struct client *c, int status_only)
+screen_redraw_screen(struct client *c, int status_only, int borders_only)
 {
 	struct window		*w = c->session->curw->window;
 	struct tty		*tty = &c->tty;
 	struct window_pane	*wp;
+	struct grid_cell	 active_gc, other_gc;
 	u_int		 	 i, j, type;
-	int		 	 status;
+	int		 	 status, fg, bg;
 	const u_char		*base, *ptr;
 	u_char		       	 ch, border[20];
 
@@ -178,8 +189,20 @@ screen_redraw_screen(struct client *c, int status_only)
 		return;
 	}
 
+	/* Set up pane border attributes. */
+	memcpy(&other_gc, &grid_default_cell, sizeof other_gc);
+	memcpy(&active_gc, &grid_default_cell, sizeof active_gc);
+	active_gc.data = other_gc.data = 'x'; /* not space */
+	fg = options_get_number(&c->session->options, "pane-border-fg");
+	colour_set_fg(&other_gc, fg);
+	bg = options_get_number(&c->session->options, "pane-border-bg");
+	colour_set_bg(&other_gc, bg);
+	fg = options_get_number(&c->session->options, "pane-active-border-fg");
+	colour_set_fg(&active_gc, fg);
+	bg = options_get_number(&c->session->options, "pane-active-border-bg");
+	colour_set_bg(&active_gc, bg);
+
 	/* Draw background and borders. */
-	tty_reset(tty);
 	strlcpy(border, " |-....--||+.", sizeof border);
 	if (tty_term_has(tty->term, TTYC_ACSC)) {
 		base = " xqlkmjwvtun~";
@@ -187,22 +210,30 @@ screen_redraw_screen(struct client *c, int status_only)
 			if ((ch = tty_get_acs(tty, *ptr)) != '\0')
 				border[ptr - base] = ch;
 		}
-		tty_putcode(tty, TTYC_SMACS);
+		other_gc.attr |= GRID_ATTR_CHARSET;
+		active_gc.attr |= GRID_ATTR_CHARSET;
 	}
 	for (j = 0; j < tty->sy - status; j++) {
 		if (status_only && j != tty->sy - 1)
 			continue;
 		for (i = 0; i < tty->sx; i++) {
 			type = screen_redraw_check_cell(c, i, j);
-			if (type != CELL_INSIDE) {
-				tty_cursor(tty, i, j);
-				tty_putc(tty, border[type]);
-			}
+			if (type == CELL_INSIDE)
+				continue;
+			if (screen_redraw_cell_border1(w->active, i, j) == 1)
+				tty_attributes(tty, &active_gc);
+			else
+				tty_attributes(tty, &other_gc);
+			tty_cursor(tty, i, j);
+			tty_putc(tty, border[type]);
 		}
 	}
-	tty_putcode(tty, TTYC_RMACS);
 
-	/* Draw the panes. */
+	/* If only drawing borders, that's it. */
+	if (borders_only)
+		return;
+
+	/* Draw the panes, if necessary. */
 	TAILQ_FOREACH(wp, &w->panes, entry) {
 		if (!window_pane_visible(wp))
 			continue;
@@ -238,18 +269,21 @@ screen_redraw_draw_number(struct client *c, struct window_pane *wp)
 {
 	struct tty		*tty = &c->tty;
 	struct session		*s = c->session;
+	struct options	        *oo = &s->options;
+	struct window		*w = wp->window;
 	struct grid_cell	 gc;
 	u_int			 idx, px, py, i, j, xoff, yoff;
-	int			 colour;
+	int			 colour, active_colour;
 	char			 buf[16], *ptr;
 	size_t			 len;
 
-	idx = window_pane_index(wp->window, wp);
+	idx = window_pane_index(w, wp);
 	len = xsnprintf(buf, sizeof buf, "%u", idx);
 
 	if (wp->sx < len)
 		return;
-	colour = options_get_number(&s->options, "display-panes-colour");
+	colour = options_get_number(oo, "display-panes-colour");
+	active_colour = options_get_number(oo, "display-panes-active-colour");
 
 	px = wp->sx / 2; py = wp->sy / 2;
 	xoff = wp->xoff; yoff = wp->yoff;
@@ -258,24 +292,30 @@ screen_redraw_draw_number(struct client *c, struct window_pane *wp)
 		tty_cursor(tty, xoff + px - len / 2, yoff + py);
 		memcpy(&gc, &grid_default_cell, sizeof gc);
 		gc.data = '_'; /* not space */
-		colour_set_fg(&gc, colour);
+		if (w->active == wp)
+			colour_set_fg(&gc, active_colour);
+		else
+			colour_set_fg(&gc, colour);
 		tty_attributes(tty, &gc);
 		tty_puts(tty, buf);
 		return;
 	}
-	
+
 	px -= len * 3;
 	py -= 2;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
 	gc.data = '_'; /* not space */
-	colour_set_bg(&gc, colour);
+	if (w->active == wp)
+		colour_set_bg(&gc, active_colour);
+	else
+		colour_set_bg(&gc, colour);
 	tty_attributes(tty, &gc);
 	for (ptr = buf; *ptr != '\0'; ptr++) {
 		if (*ptr < '0' || *ptr > '9')
 			continue;
 		idx = *ptr - '0';
-		
+
 		for (j = 0; j < 5; j++) {
 			for (i = px; i < px + 5; i++) {
 				tty_cursor(tty, xoff + i, yoff + py + j);
