@@ -1,4 +1,4 @@
-/* $Id: screen-write.c,v 1.83 2009/10/23 17:16:24 tcunha Exp $ */
+/* $Id: screen-write.c,v 1.90 2010/06/16 18:09:23 micahcowan Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -23,7 +23,7 @@
 #include "tmux.h"
 
 void	screen_write_initctx(struct screen_write_ctx *, struct tty_ctx *, int);
-void	screen_write_overwrite(struct screen_write_ctx *);
+void	screen_write_overwrite(struct screen_write_ctx *, u_int);
 int	screen_write_combine(
 	    struct screen_write_ctx *, const struct utf8_data *);
 
@@ -40,6 +40,7 @@ screen_write_start(
 }
 
 /* Finish writing. */
+/* ARGSUSED */
 void
 screen_write_stop(unused struct screen_write_ctx *ctx)
 {
@@ -182,7 +183,7 @@ screen_write_vnputs(struct screen_write_ctx *ctx, ssize_t maxlen,
 				break;
 			}
 			size += utf8data.width;
-			
+
 			gc->flags |= GRID_FLAG_UTF8;
 			screen_write_cell(ctx, gc, &utf8data);
 			gc->flags &= ~GRID_FLAG_UTF8;
@@ -354,7 +355,7 @@ screen_write_copy(struct screen_write_ctx *ctx,
 	const struct grid_cell	*gc;
 	const struct grid_utf8	*gu;
 	struct utf8_data	 utf8data;
-	u_int		 	 xx, yy, cx, cy, ax, bx, i;
+	u_int		 	 xx, yy, cx, cy, ax, bx;
 
 	cx = s->cx;
 	cy = s->cy;
@@ -375,24 +376,21 @@ screen_write_copy(struct screen_write_ctx *ctx,
 				bx = gl->cellsize;
 			else
 				bx = px + nx;
-			
+
 			for (xx = ax; xx < bx; xx++) {
 				if (xx >= gl->cellsize)
 					gc = &grid_default_cell;
 				else
 					gc = &gl->celldata[xx];
-				if (gc->flags & GRID_FLAG_UTF8) {
-					gu = &gl->utf8data[xx]; 
-					memcpy(utf8data.data,
-					    gu->data, sizeof utf8data.data);
-					utf8data.width = gu->width;
-					utf8data.size = 0;
-					for (i = 0; i < UTF8_SIZE; i++) {
-						if (gu->data[i] == 0xff)
-							break;
-						utf8data.size++;
-					}
+				if (!(gc->flags & GRID_FLAG_UTF8)) {
+					screen_write_cell(ctx, gc, NULL);
+					continue;
 				}
+				/* Reinject the UTF-8 sequence. */
+				gu = &gl->utf8data[xx];
+				utf8data.size = grid_utf8_copy(
+				    gu, utf8data.data, sizeof utf8data.data);
+				utf8data.width = gu->width;
 				screen_write_cell(ctx, gc, &utf8data);
 			}
 			if (px + nx == gd->sx && px + nx > gl->cellsize)
@@ -427,8 +425,8 @@ screen_write_initctx(
 		return;
 
 	/* Save the last cell on the screen. */
-	gc = NULL;
-	for (xx = 1; xx < screen_size_x(s); xx++) {
+	gc = &grid_default_cell;
+	for (xx = 1; xx <= screen_size_x(s); xx++) {
 		gc = grid_view_peek_cell(gd, screen_size_x(s) - xx, s->cy);
 		if (!(gc->flags & GRID_FLAG_PADDING))
 			break;
@@ -648,7 +646,7 @@ screen_write_insertline(struct screen_write_ctx *ctx, u_int ny)
 		ny = s->rlower + 1 - s->cy;
 	if (ny == 0)
 		return;
-	
+
 	screen_write_initctx(ctx, &ttyctx, 0);
 
 	if (s->cy < s->rupper || s->cy > s->rlower)
@@ -684,7 +682,7 @@ screen_write_deleteline(struct screen_write_ctx *ctx, u_int ny)
 		tty_write(tty_cmd_deleteline, &ttyctx);
 		return;
 	}
-	
+
 	if (ny > s->rlower + 1 - s->cy)
 		ny = s->rlower + 1 - s->cy;
 	if (ny == 0)
@@ -730,7 +728,7 @@ screen_write_clearendofline(struct screen_write_ctx *ctx)
 	if (s->cx <= sx - 1)
 		grid_view_clear(s->grid, s->cx, s->cy, sx - s->cx, 1);
 
- 	tty_write(tty_cmd_clearendofline, &ttyctx);
+	tty_write(tty_cmd_clearendofline, &ttyctx);
 }
 
 /* Clear to start of line from cursor. */
@@ -865,7 +863,7 @@ screen_write_linefeed(struct screen_write_ctx *ctx, int wrapped)
 		s->cy++;
 
 	ttyctx.num = wrapped;
- 	tty_write(tty_cmd_linefeed, &ttyctx);
+	tty_write(tty_cmd_linefeed, &ttyctx);
 }
 
 /* Carriage return (cursor to start of line). */
@@ -990,7 +988,7 @@ screen_write_cell(struct screen_write_ctx *ctx,
 
 	/*
 	 * If the width is zero, combine onto the previous character, if
-	 * there is space. 
+	 * there is space.
 	 */
 	if (width == 0) {
 		if (screen_write_combine(ctx, utf8data) == 0) {
@@ -1011,17 +1009,18 @@ screen_write_cell(struct screen_write_ctx *ctx,
 	}
 
 	/* Check this will fit on the current line and wrap if not. */
-	if (s->cx > screen_size_x(s) - width) {
+	if ((s->mode & MODE_WRAP) && s->cx > screen_size_x(s) - width) {
 		screen_write_linefeed(ctx, 1);
 		s->cx = 0;	/* carriage return */
 	}
 
 	/* Sanity checks. */
-	if (s->cx > screen_size_x(s) - 1 || s->cy > screen_size_y(s) - 1)
+	if (((s->mode & MODE_WRAP) && s->cx > screen_size_x(s) - 1)
+	    || s->cy > screen_size_y(s) - 1)
 		return;
 
 	/* Handle overwriting of UTF-8 characters. */
-	screen_write_overwrite(ctx);
+	screen_write_overwrite(ctx, width);
 
 	/*
 	 * If the new character is UTF-8 wide, fill in padding cells. Have
@@ -1037,11 +1036,7 @@ screen_write_cell(struct screen_write_ctx *ctx,
 	grid_view_set_cell(gd, s->cx, s->cy, gc);
 	if (gc->flags & GRID_FLAG_UTF8) {
 		/* Construct UTF-8 and write it. */
-		gu.width = utf8data->width;
-		memset(gu.data, 0xff, sizeof gu.data);
-		if (utf8data->size > sizeof gu.data)
-			fatalx("UTF-8 data overflow");
-		memcpy(gu.data, utf8data->data, utf8data->size);
+		grid_utf8_set(&gu, utf8data);
 		grid_view_set_utf8(gd, s->cx, s->cy, &gu);
 	}
 
@@ -1078,43 +1073,43 @@ screen_write_combine(
 	struct grid		*gd = s->grid;
 	struct grid_cell	*gc;
 	struct grid_utf8	*gu, tmp_gu;
-	u_int			 i, old_size;
+	u_int			 i;
 
 	/* Can't combine if at 0. */
 	if (s->cx == 0)
 		return (-1);
 
+	/* Empty utf8data is out. */
+	if (utf8data->size == 0)
+		fatalx("UTF-8 data empty");
+
 	/* Retrieve the previous cell and convert to UTF-8 if not already. */
 	gc = grid_view_get_cell(gd, s->cx - 1, s->cy);
 	if (!(gc->flags & GRID_FLAG_UTF8)) {
-		memset(&tmp_gu.data, 0xff, sizeof tmp_gu.data);
-		*tmp_gu.data = gc->data;
+		tmp_gu.data[0] = gc->data;
+		tmp_gu.data[1] = 0xff;
 		tmp_gu.width = 1;
 
 		grid_view_set_utf8(gd, s->cx - 1, s->cy, &tmp_gu);
 		gc->flags |= GRID_FLAG_UTF8;
 	}
 
-	/* Get the previous cell's UTF-8 data and its size. */
+	/* Append the current cell. */
 	gu = grid_view_get_utf8(gd, s->cx - 1, s->cy);
-	for (old_size = 0; old_size < UTF8_SIZE; old_size++) {
-		if (gu->data[old_size] == 0xff)
-			break;
+	if (grid_utf8_append(gu, utf8data) != 0) {
+		/* Failed: scrap this character and replace with underscores. */
+		if (gu->width == 1) {
+			gc->data = '_';
+			gc->flags &= ~GRID_FLAG_UTF8;
+		} else {
+			for (i = 0; i < gu->width && i != sizeof gu->data; i++)
+				gu->data[i] = '_';
+			if (i != sizeof gu->data)
+				gu->data[i] = 0xff;
+			gu->width = i;
+		}
 	}
 
-	/* If there isn't space, scrap this character. */
-	if (old_size + utf8data->size > UTF8_SIZE) {
-		for (i = 0; i < gu->width && i != UTF8_SIZE; i++)
-			gu->data[i] = '_';
-		if (i != UTF8_SIZE)
-			gu->data[i] = 0xff;
-		return (0);
-	}
-
-	/* Otherwise save the character. */
-	memcpy(gu->data + old_size, utf8data->data, utf8data->size);
-	if (old_size + utf8data->size != UTF8_SIZE)
-		gu->data[old_size + utf8data->size] = 0xff;
 	return (0);
 }
 
@@ -1128,12 +1123,11 @@ screen_write_combine(
  * by the same character.
  */
 void
-screen_write_overwrite(struct screen_write_ctx *ctx)
+screen_write_overwrite(struct screen_write_ctx *ctx, u_int width)
 {
 	struct screen		*s = ctx->s;
 	struct grid		*gd = s->grid;
 	const struct grid_cell	*gc;
-	const struct grid_utf8	*gu;
 	u_int			 xx;
 
 	gc = grid_view_peek_cell(gd, s->cx, s->cy);
@@ -1153,28 +1147,17 @@ screen_write_overwrite(struct screen_write_ctx *ctx)
 
 		/* Overwrite the character at the start of this padding. */
 		grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
+	}
 
-		/* Overwrite following padding cells. */
-		xx = s->cx;
-		while (++xx < screen_size_x(s)) {
-			gc = grid_view_peek_cell(gd, xx, s->cy);
-			if (!(gc->flags & GRID_FLAG_PADDING))
-				break;
-			grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
-		}
-	} else if (gc->flags & GRID_FLAG_UTF8) {
-		gu = grid_view_peek_utf8(gd, s->cx, s->cy);
-		if (gu->width > 1) {
-			/*
-			 * An UTF-8 wide cell; overwrite following padding cells only.
-			 */
-			xx = s->cx;
-			while (++xx < screen_size_x(s)) {
-				gc = grid_view_peek_cell(gd, xx, s->cy);
-				if (!(gc->flags & GRID_FLAG_PADDING))
-					break;
-				grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
-			}
-		}
+	/*
+	 * Overwrite any padding cells that belong to a UTF-8 character
+	 * we'll be overwriting with the current character.
+	 */
+	xx = s->cx + width - 1;
+	while (++xx < screen_size_x(s)) {
+		gc = grid_view_peek_cell(gd, xx, s->cy);
+		if (!(gc->flags & GRID_FLAG_PADDING))
+			break;
+		grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 	}
 }
