@@ -1,4 +1,4 @@
-/* $Id: server-fn.c,v 1.94 2009/10/12 00:37:41 tcunha Exp $ */
+/* $Id: server-fn.c,v 1.108 2010/07/02 02:45:52 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,6 +24,9 @@
 
 #include "tmux.h"
 
+struct session *server_next_session(struct session *);
+void		server_callback_identify(int, short, void *);
+
 void
 server_fill_environ(struct session *s, struct environ *env)
 {
@@ -40,15 +43,6 @@ server_fill_environ(struct session *s, struct environ *env)
 	environ_set(env, "TERM", term);
 }
 
-void
-server_write_error(struct client *c, const char *msg)
-{
-	struct msg_print_data	printdata;
-
-	strlcpy(printdata.msg, msg, sizeof printdata.msg);
-	server_write_client(c, MSG_ERROR, &printdata, sizeof printdata);
-}
-
 void
 server_write_client(
     struct client *c, enum msgtype type, const void *buf, size_t len)
@@ -59,6 +53,7 @@ server_write_client(
 		return;
 	log_debug("writing %d to client %d", type, c->ibuf.fd);
 	imsg_compose(ibuf, type, PROTOCOL_VERSION, -1, -1, (void *) buf, len);
+	server_update_event(c);
 }
 
 void
@@ -161,6 +156,21 @@ server_redraw_window(struct window *w)
 	w->flags |= WINDOW_REDRAW;
 }
 
+void
+server_redraw_window_borders(struct window *w)
+{
+	struct client	*c;
+	u_int		 i;
+
+	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+		c = ARRAY_ITEM(&clients, i);
+		if (c == NULL || c->session == NULL)
+			continue;
+		if (c->session->curw->window == w)
+			c->flags |= CLIENT_BORDERS;
+	}
+}
+
 void
 server_status_window(struct window *w)
 {
@@ -175,7 +185,7 @@ server_status_window(struct window *w)
 
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
-		if (s != NULL && session_has(s, w))
+		if (s != NULL && session_has(s, w) != NULL)
 			server_status_session(s);
 	}
 }
@@ -205,7 +215,7 @@ server_lock_session(struct session *s)
 		if (c == NULL || c->session == NULL || c->session != s)
 			continue;
 		server_lock_client(c);
-	}	
+	}
 }
 
 void
@@ -222,7 +232,7 @@ server_lock_client(struct client *c)
 	cmdlen = strlcpy(lockdata.cmd, cmd, sizeof lockdata.cmd);
 	if (cmdlen >= sizeof lockdata.cmd)
 		return;
-      
+
 	tty_stop_tty(&c->tty);
 	tty_raw(&c->tty, tty_term_string(c->tty.term, TTYC_SMCUP));
 	tty_raw(&c->tty, tty_term_string(c->tty.term, TTYC_CLEAR));
@@ -237,19 +247,17 @@ server_kill_window(struct window *w)
 	struct session	*s;
 	struct winlink	*wl;
 	u_int		 i;
-	
+
 	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
 		s = ARRAY_ITEM(&sessions, i);
-		if (s == NULL || !session_has(s, w))
+		if (s == NULL || session_has(s, w) == NULL)
 			continue;
-		if ((wl = winlink_find_by_window(&s->windows, w)) == NULL)
-			continue;
-		
-		if (session_detach(s, wl))
-			server_destroy_session_group(s);
-		else {
-			server_redraw_session(s);
-			server_status_session_group(s);
+		while ((wl = winlink_find_by_window(&s->windows, w)) != NULL) {
+			if (session_detach(s, wl)) {
+				server_destroy_session_group(s);
+				break;
+			} else
+				server_redraw_session_group(s);
 		}
 	}
 }
@@ -279,7 +287,7 @@ server_link_window(struct session *src, struct winlink *srcwl,
 			 * Can't use session_detach as it will destroy session
 			 * if this makes it empty.
 			 */
-			session_alert_cancel(dst, dstwl);
+			dstwl->flags &= ~WINLINK_ALERTFLAGS;
 			winlink_stack_remove(&dst->lastw, dstwl);
 			winlink_remove(&dst->windows, dstwl);
 
@@ -313,6 +321,29 @@ server_unlink_window(struct session *s, struct winlink *wl)
 		server_redraw_session_group(s);
 }
 
+void
+server_destroy_pane(struct window_pane *wp)
+{
+	struct window	*w = wp->window;
+
+	if (wp->fd != -1) {
+		close(wp->fd);
+		bufferevent_free(wp->event);
+		wp->fd = -1;
+	}
+
+	if (options_get_number(&w->options, "remain-on-exit"))
+		return;
+
+	layout_close_pane(wp);
+	window_remove_pane(w, wp);
+
+	if (TAILQ_EMPTY(&w->panes))
+		server_kill_window(w);
+	else
+		server_redraw_window(w);
+}
+
 void
 server_destroy_session_group(struct session *s)
 {
@@ -328,19 +359,49 @@ server_destroy_session_group(struct session *s)
 	}
 }
 
+struct session *
+server_next_session(struct session *s)
+{
+	struct session *s_loop, *s_out;
+	u_int		i;
+
+	s_out = NULL;
+	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
+		s_loop = ARRAY_ITEM(&sessions, i);
+		if (s_loop == s)
+			continue;
+		if (s_out == NULL ||
+		    timercmp(&s_loop->activity_time, &s_out->activity_time, <))
+			s_out = s_loop;
+	}
+	return (s_out);
+}
+
 void
 server_destroy_session(struct session *s)
 {
 	struct client	*c;
+	struct session	*s_new;
 	u_int		 i;
-	
+
+	if (!options_get_number(&s->options, "detach-on-destroy"))
+		s_new = server_next_session(s);
+	else
+		s_new = NULL;
+
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 		if (c == NULL || c->session != s)
 			continue;
-		c->session = NULL;
-		server_write_client(c, MSG_EXIT, NULL, 0);
+		if (s_new == NULL) {
+			c->session = NULL;
+			server_write_client(c, MSG_EXIT, NULL, 0);
+		} else {
+			c->session = s_new;
+			server_redraw_client(c);
+		}
 	}
+	recalculate_sizes();
 }
 
 void
@@ -353,9 +414,9 @@ server_set_identify(struct client *c)
 	tv.tv_sec = delay / 1000;
 	tv.tv_usec = (delay % 1000) * 1000L;
 
-	if (gettimeofday(&c->identify_timer, NULL) != 0)
-		fatal("gettimeofday failed");
-	timeradd(&c->identify_timer, &tv, &c->identify_timer);
+	evtimer_del(&c->identify_timer);
+	evtimer_set(&c->identify_timer, server_callback_identify, c);
+	evtimer_add(&c->identify_timer, &tv);
 
 	c->flags |= CLIENT_IDENTIFY;
 	c->tty.flags |= (TTY_FREEZE|TTY_NOCURSOR);
@@ -371,3 +432,27 @@ server_clear_identify(struct client *c)
 		server_redraw_client(c);
 	}
 }
+
+/* ARGSUSED */
+void
+server_callback_identify(unused int fd, unused short events, void *data)
+{
+	struct client	*c = data;
+
+	server_clear_identify(c);
+}
+
+void
+server_update_event(struct client *c)
+{
+	short	events;
+
+	events = 0;
+	if (!(c->flags & CLIENT_BAD))
+		events |= EV_READ;
+	if (c->ibuf.w.queued > 0)
+		events |= EV_WRITE;
+	event_del(&c->event);
+	event_set(&c->event, c->ibuf.fd, events, server_client_callback, c);
+	event_add(&c->event, NULL);
+}
