@@ -1,4 +1,4 @@
-/* $Id: server-window.c,v 1.4 2009/11/04 22:47:29 tcunha Exp $ */
+/* $Id: server-window.c,v 1.15 2010/06/22 23:26:18 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,51 +18,16 @@
 
 #include <sys/types.h>
 
+#include <event.h>
 #include <unistd.h>
 
 #include "tmux.h"
 
 int	server_window_backoff(struct window_pane *);
-int	server_window_check_bell(struct session *, struct window *);
-int	server_window_check_activity(struct session *, struct window *);
+int	server_window_check_bell(struct session *, struct winlink *);
+int	server_window_check_activity(struct session *, struct winlink *);
 int	server_window_check_content(
-	    struct session *, struct window *, struct window_pane *);
-void	server_window_check_alive(struct window *);
-
-/* Register windows for poll. */
-void
-server_window_prepare(void)
-{
-	struct window		*w;
-	struct window_pane	*wp;
-	u_int		 	 i;
-	int			 events;
-
-	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
-		if ((w = ARRAY_ITEM(&windows, i)) == NULL)
-			continue;
-
-		TAILQ_FOREACH(wp, &w->panes, entry) {	
-			if (wp->fd == -1)
-				continue;
-			events = 0;
-			if (!server_window_backoff(wp))
-				events |= POLLIN;
-			if (BUFFER_USED(wp->out) > 0)
-				events |= POLLOUT;
-			server_poll_add(
-			    wp->fd, events, server_window_callback, wp);
-
-			if (wp->pipe_fd == -1)
-				continue;
-			events = 0;
-			if (BUFFER_USED(wp->pipe_buf) > 0)
-				events |= POLLOUT;
-			server_poll_add(
-			    wp->pipe_fd, events, server_window_callback, wp);
-		}
-	}
-}
+	    struct session *, struct winlink *, struct window_pane *);
 
 /* Check if this window should suspend reading. */
 int
@@ -82,43 +47,19 @@ server_window_backoff(struct window_pane *wp)
 			continue;
 		if (c->session->curw->window != wp->window)
 			continue;
-		if (BUFFER_USED(c->tty.out) > BACKOFF_THRESHOLD)
+
+		if (EVBUFFER_LENGTH(c->tty.event->output) > BACKOFF_THRESHOLD)
 			return (1);
 	}
 	return (0);
 }
 
-/* Process a single window pane event. */
-void
-server_window_callback(int fd, int events, void *data)
-{
-	struct window_pane	*wp = data;
-
-	if (wp->fd == -1)
-		return;
-
-	if (fd == wp->fd) {
-		if (buffer_poll(fd, events, wp->in, wp->out) != 0) {
-			close(wp->fd);
-			wp->fd = -1;
-		} else
-			window_pane_parse(wp);
-	}
-
-	if (fd == wp->pipe_fd) {
-		if (buffer_poll(fd, events, NULL, wp->pipe_buf) != 0) {
-			buffer_destroy(wp->pipe_buf);
-			close(wp->pipe_fd);
-			wp->pipe_fd = -1;
-		}
-	}
-}
-
 /* Window functions that need to happen every loop. */
 void
 server_window_loop(void)
 {
 	struct window		*w;
+	struct winlink		*wl;
 	struct window_pane	*wp;
 	struct session		*s;
 	u_int		 	 i, j;
@@ -128,39 +69,50 @@ server_window_loop(void)
 		if (w == NULL)
 			continue;
 
+		TAILQ_FOREACH(wp, &w->panes, entry) {
+			if (wp->fd == -1)
+				continue;
+			if (!(wp->flags & PANE_FREEZE)) {
+				if (server_window_backoff(wp))
+					bufferevent_disable(wp->event, EV_READ);
+				else
+					bufferevent_enable(wp->event, EV_READ);
+			}
+		}
+
 		for (j = 0; j < ARRAY_LENGTH(&sessions); j++) {
 			s = ARRAY_ITEM(&sessions, j);
-			if (s == NULL || !session_has(s, w))
+			if (s == NULL)
 				continue;
-			
-			if (server_window_check_bell(s, w) ||
-			    server_window_check_activity(s, w))
+			wl = session_has(s, w);
+			if (wl == NULL)
+				continue;
+
+			if (server_window_check_bell(s, wl) ||
+			    server_window_check_activity(s, wl))
 				server_status_session(s);
 			TAILQ_FOREACH(wp, &w->panes, entry)
-				server_window_check_content(s, w, wp);
+				server_window_check_content(s, wl, wp);
 		}
-		w->flags &= ~(WINDOW_BELL|WINDOW_ACTIVITY|WINDOW_CONTENT);
-
-		server_window_check_alive(w);
+		w->flags &= ~(WINDOW_BELL|WINDOW_ACTIVITY);
 	}
-
-	set_window_names();
 }
 
 /* Check for bell in window. */
 int
-server_window_check_bell(struct session *s, struct window *w)
+server_window_check_bell(struct session *s, struct winlink *wl)
 {
 	struct client	*c;
+	struct window	*w = wl->window;
 	u_int		 i;
 	int		 action, visual;
 
-	if (!(w->flags & WINDOW_BELL))
+	if (!(w->flags & WINDOW_BELL) || wl->flags & WINLINK_BELL)
+		return (0);
+	if (s->curw == wl)
 		return (0);
 
-	if (session_alert_has_window(s, w, WINDOW_BELL))
-		return (0);
-	session_alert_add(s, w, WINDOW_BELL);
+	wl->flags |= WINLINK_BELL;
 
 	action = options_get_number(&s->options, "bell-action");
 	switch (action) {
@@ -176,7 +128,7 @@ server_window_check_bell(struct session *s, struct window *w)
 				tty_putcode(&c->tty, TTYC_BEL);
 				continue;
 			}
- 			if (c->session->curw->window == w) {
+			if (c->session->curw->window == w) {
 				status_message_set(c, "Bell in current window");
 				continue;
 			}
@@ -192,7 +144,7 @@ server_window_check_bell(struct session *s, struct window *w)
 			c = ARRAY_ITEM(&clients, i);
 			if (c == NULL || c->session != s)
 				continue;
- 			if (c->session->curw->window != w)
+			if (c->session->curw->window != w)
 				continue;
 			if (!visual) {
 				tty_putcode(&c->tty, TTYC_BEL);
@@ -208,26 +160,23 @@ server_window_check_bell(struct session *s, struct window *w)
 
 /* Check for activity in window. */
 int
-server_window_check_activity(struct session *s, struct window *w)
+server_window_check_activity(struct session *s, struct winlink *wl)
 {
 	struct client	*c;
+	struct window	*w = wl->window;
 	u_int		 i;
 
-	if (!(w->flags & WINDOW_ACTIVITY))
+	if (!(w->flags & WINDOW_ACTIVITY) || wl->flags & WINLINK_ACTIVITY)
 		return (0);
-	if (s->curw->window == w)
+	if (s->curw == wl)
 		return (0);
 
- 	if (!options_get_number(&w->options, "monitor-activity"))
+	if (!options_get_number(&w->options, "monitor-activity"))
 		return (0);
 
-	if (session_alert_has_window(s, w, WINDOW_ACTIVITY))
-		return (0);
-	session_alert_add(s, w, WINDOW_ACTIVITY);
+	wl->flags |= WINLINK_ACTIVITY;
 
-	if (s->flags & SESSION_UNATTACHED)
-		return (0);
- 	if (options_get_number(&s->options, "visual-activity")) {
+	if (options_get_number(&s->options, "visual-activity")) {
 		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 			c = ARRAY_ITEM(&clients, i);
 			if (c == NULL || c->session != s)
@@ -243,32 +192,29 @@ server_window_check_activity(struct session *s, struct window *w)
 /* Check for content change in window. */
 int
 server_window_check_content(
-    struct session *s, struct window *w, struct window_pane *wp)
+    struct session *s, struct winlink *wl, struct window_pane *wp)
 {
 	struct client	*c;
+	struct window	*w = wl->window;
 	u_int		 i;
 	char		*found, *ptr;
-	
-	if (!(w->flags & WINDOW_ACTIVITY))	/* activity for new content */
+
+	/* Activity flag must be set for new content. */
+	if (!(w->flags & WINDOW_ACTIVITY) || wl->flags & WINLINK_CONTENT)
 		return (0);
-	if (s->curw->window == w)
+	if (s->curw == wl)
 		return (0);
 
 	ptr = options_get_string(&w->options, "monitor-content");
 	if (ptr == NULL || *ptr == '\0')
 		return (0);
-
-	if (session_alert_has_window(s, w, WINDOW_CONTENT))
-		return (0);
-
 	if ((found = window_pane_search(wp, ptr, NULL)) == NULL)
 		return (0);
-    	xfree(found);
+	xfree(found);
 
-	session_alert_add(s, w, WINDOW_CONTENT);
-	if (s->flags & SESSION_UNATTACHED)
-		return (0);
- 	if (options_get_number(&s->options, "visual-content")) {
+	wl->flags |= WINLINK_CONTENT;
+
+	if (options_get_number(&s->options, "visual-content")) {
 		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 			c = ARRAY_ITEM(&clients, i);
 			if (c == NULL || c->session != s)
@@ -280,62 +226,3 @@ server_window_check_content(
 
 	return (1);
 }
-
-/* Check if window still exists. */
-void
-server_window_check_alive(struct window *w)
-{
-	struct window_pane	*wp, *wq;
-	struct options		*oo = &w->options;
-	struct session		*s;
-	struct winlink		*wl;
-	u_int		 	 i;
-	int		 	 destroyed;
-
-	destroyed = 1;
-
-	wp = TAILQ_FIRST(&w->panes);
-	while (wp != NULL) {
-		wq = TAILQ_NEXT(wp, entry);
-		/*
-		 * If the pane has died and the remain-on-exit flag is not set,
-		 * remove the pane; otherwise, if the flag is set, don't allow
-		 * the window to be destroyed (or it'll close when the last
-		 * pane dies).
-		 */
-		if (wp->fd == -1 && !options_get_number(oo, "remain-on-exit")) {
-			layout_close_pane(wp);
-			window_remove_pane(w, wp);
-			server_redraw_window(w);
-		} else 
-			destroyed = 0;
-		wp = wq;
-	}
-
-	if (!destroyed)
-		return;
-
-	for (i = 0; i < ARRAY_LENGTH(&sessions); i++) {
-		s = ARRAY_ITEM(&sessions, i);
-		if (s == NULL)
-			continue;
-		if (!session_has(s, w))
-			continue;
-
-	restart:
-		/* Detach window and either redraw or kill clients. */
-		RB_FOREACH(wl, winlinks, &s->windows) {
-			if (wl->window != w)
-				continue;
-			if (session_detach(s, wl)) {
-				server_destroy_session_group(s);
-				break;
-			}
-			server_redraw_session(s);
-			server_status_session_group(s);
-			goto restart;
-		}
-	}
-
-	recalculate_sizes();
-}
