@@ -1,4 +1,4 @@
-/* $Id: status.c,v 1.125 2009/11/04 23:12:32 tcunha Exp $ */
+/* $Id: status.c,v 1.149 2010/06/22 23:26:18 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,281 +29,293 @@
 
 #include "tmux.h"
 
+char   *status_redraw_get_left(
+	    struct client *, time_t, int, struct grid_cell *, size_t *);
+char   *status_redraw_get_right(
+	    struct client *, time_t, int, struct grid_cell *, size_t *);
 char   *status_job(struct client *, char **);
 void	status_job_callback(struct job *);
-size_t	status_width(struct winlink *);
-char   *status_print(struct session *, struct winlink *, struct grid_cell *);
+char   *status_print(
+	    struct client *, struct winlink *, time_t, struct grid_cell *);
+void	status_replace1(struct client *,
+	    struct winlink *, char **, char **, char *, size_t, int);
+void	status_message_callback(int, short, void *);
 
 void	status_prompt_add_history(struct client *);
 char   *status_prompt_complete(const char *);
 
+/* Retrieve options for left string. */
+char *
+status_redraw_get_left(struct client *c,
+    time_t t, int utf8flag, struct grid_cell *gc, size_t *size)
+{
+	struct session	*s = c->session;
+	char		*left;
+	u_char		 fg, bg, attr;
+	size_t		 leftlen;
+
+	fg = options_get_number(&s->options, "status-left-fg");
+	if (fg != 8)
+		colour_set_fg(gc, fg);
+	bg = options_get_number(&s->options, "status-left-bg");
+	if (bg != 8)
+		colour_set_bg(gc, bg);
+	attr = options_get_number(&s->options, "status-left-attr");
+	if (attr != 0)
+		gc->attr = attr;
+
+	left = status_replace(
+	    c, NULL, options_get_string(&s->options, "status-left"), t, 1);
+
+	*size = options_get_number(&s->options, "status-left-length");
+	leftlen = screen_write_cstrlen(utf8flag, "%s", left);
+	if (leftlen < *size)
+		*size = leftlen;
+	return (left);
+}
+
+/* Retrieve options for right string. */
+char *
+status_redraw_get_right(struct client *c,
+    time_t t, int utf8flag, struct grid_cell *gc, size_t *size)
+{
+	struct session	*s = c->session;
+	char		*right;
+	u_char		 fg, bg, attr;
+	size_t		 rightlen;
+
+	fg = options_get_number(&s->options, "status-right-fg");
+	if (fg != 8)
+		colour_set_fg(gc, fg);
+	bg = options_get_number(&s->options, "status-right-bg");
+	if (bg != 8)
+		colour_set_bg(gc, bg);
+	attr = options_get_number(&s->options, "status-right-attr");
+	if (attr != 0)
+		gc->attr = attr;
+
+	right = status_replace(
+	    c, NULL, options_get_string(&s->options, "status-right"), t, 1);
+
+	*size = options_get_number(&s->options, "status-right-length");
+	rightlen = screen_write_cstrlen(utf8flag, "%s", right);
+	if (rightlen < *size)
+		*size = rightlen;
+	return (right);
+}
+
 /* Draw status for client on the last lines of given context. */
 int
 status_redraw(struct client *c)
 {
-	struct screen_write_ctx		ctx;
-	struct session		       *s = c->session;
-	struct winlink		       *wl;
-	struct screen		      	old_status;
-	char		 	       *left, *right, *text, *ptr;
-	size_t				llen, llen2, rlen, rlen2, offset;
-	size_t				ox, xx, yy, size, start, width;
-	struct grid_cell	        stdgc, sl_stdgc, sr_stdgc, gc;
-	int				larrow, rarrow, utf8flag;
-	int				sl_fg, sl_bg, sr_fg, sr_bg;
-	int				sl_attr, sr_attr;
+	struct screen_write_ctx	ctx;
+	struct session	       *s = c->session;
+	struct winlink	       *wl;
+	struct screen		old_status, window_list;
+	struct grid_cell	stdgc, lgc, rgc, gc;
+	time_t			t;
+	char		       *left, *right;
+	u_int			offset, needed;
+	u_int			wlstart, wlwidth, wlavailable, wloffset, wlsize;
+	size_t			llen, rlen;
+	int			larrow, rarrow, utf8flag;
 
-	left = right = NULL;
-
-	/* No status line?*/
+	/* No status line? */
 	if (c->tty.sy == 0 || !options_get_number(&s->options, "status"))
 		return (1);
+	left = right = NULL;
 	larrow = rarrow = 0;
 
-	/* Create the target screen. */
-	memcpy(&old_status, &c->status, sizeof old_status);
-	screen_init(&c->status, c->tty.sx, 1, 0);
-
+	/* Update status timer. */
 	if (gettimeofday(&c->status_timer, NULL) != 0)
 		fatal("gettimeofday failed");
+	t = c->status_timer.tv_sec;
+
+	/* Set up default colour. */
 	memcpy(&stdgc, &grid_default_cell, sizeof gc);
 	colour_set_fg(&stdgc, options_get_number(&s->options, "status-fg"));
 	colour_set_bg(&stdgc, options_get_number(&s->options, "status-bg"));
 	stdgc.attr |= options_get_number(&s->options, "status-attr");
 
-	/* 
-	 * Set the status-left and status-right parts to the default status
-	 * line options and only change them where they differ from the
-	 * defaults.
-	 */
-	memcpy(&sl_stdgc, &stdgc, sizeof sl_stdgc);
-	memcpy(&sr_stdgc, &stdgc, sizeof sr_stdgc);
-	sl_fg = options_get_number(&s->options, "status-left-fg");
-	if (sl_fg != 8)
-		colour_set_fg(&sl_stdgc, sl_fg);
-	sl_bg = options_get_number(&s->options, "status-left-bg");
-	if (sl_bg != 8)
-		colour_set_bg(&sl_stdgc, sl_bg);
-	sl_attr = options_get_number(&s->options, "status-left-attr");
-	if (sl_attr != 0)
-		sl_stdgc.attr = sl_attr;
-	sr_fg = options_get_number(&s->options, "status-right-fg");
-	if (sr_fg != 8)
-		colour_set_fg(&sr_stdgc, sr_fg);
-	sr_bg = options_get_number(&s->options, "status-right-bg");
-	if (sr_bg != 8)
-		colour_set_bg(&sr_stdgc, sr_bg);
-	sr_attr = options_get_number(&s->options, "status-right-attr");
-	if (sr_attr != 0)
-		sr_stdgc.attr = sr_attr;
+	/* Create the target screen. */
+	memcpy(&old_status, &c->status, sizeof old_status);
+	screen_init(&c->status, c->tty.sx, 1, 0);
+	screen_write_start(&ctx, NULL, &c->status);
+	for (offset = 0; offset < c->tty.sx; offset++)
+		screen_write_putc(&ctx, &stdgc, ' ');
+	screen_write_stop(&ctx);
 
-	yy = c->tty.sy - 1;
-	if (yy == 0)
-		goto blank;
+	/* If the height is one line, blank status line. */
+	if (c->tty.sy <= 1)
+		goto out;
 
-	/* Caring about UTF-8 in status line? */
+	/* Get UTF-8 flag. */
 	utf8flag = options_get_number(&s->options, "status-utf8");
 
-	/* Work out the left and right strings. */
-	left = status_replace(c, options_get_string(
-	    &s->options, "status-left"), c->status_timer.tv_sec);
-	llen = options_get_number(&s->options, "status-left-length");
-	llen2 = screen_write_cstrlen(utf8flag, "%s", left);
-	if (llen2 < llen)
-		llen = llen2;
-
-	right = status_replace(c, options_get_string(
-	    &s->options, "status-right"), c->status_timer.tv_sec);
-	rlen = options_get_number(&s->options, "status-right-length");
-	rlen2 = screen_write_cstrlen(utf8flag, "%s", right);
-	if (rlen2 < rlen)
-		rlen = rlen2;
+	/* Work out left and right strings. */
+	memcpy(&lgc, &stdgc, sizeof lgc);
+	left = status_redraw_get_left(c, t, utf8flag, &lgc, &llen);
+	memcpy(&rgc, &stdgc, sizeof rgc);
+	right = status_redraw_get_right(c, t, utf8flag, &rgc, &rlen);
 
 	/*
-	 * Figure out how much space we have for the window list. If there isn't
-	 * enough space, just wimp out.
+	 * Figure out how much space we have for the window list. If there
+	 * isn't enough space, just show a blank status line.
 	 */
-	xx = 0;
+	needed = 0;
 	if (llen != 0)
-		xx += llen + 1;
+		needed += llen + 1;
 	if (rlen != 0)
-		xx += rlen + 1;
-	if (c->tty.sx == 0 || c->tty.sx <= xx)
-		goto blank;
-	xx = c->tty.sx - xx;
+		needed += rlen + 1;
+	if (c->tty.sx == 0 || c->tty.sx <= needed)
+		goto out;
+	wlavailable = c->tty.sx - needed;
 
-	/*
-	 * Right. We have xx characters to fill. Find out how much is to go in
-	 * them and the offset of the current window (it must be on screen).
-	 */
-	width = offset = 0;
+	/* Calculate the total size needed for the window list. */
+	wlstart = wloffset = wlwidth = 0;
 	RB_FOREACH(wl, winlinks, &s->windows) {
-		size = status_width(wl) + 1;
-		if (wl == s->curw)
-			offset = width;
-		width += size;
-	}
-	start = 0;
+		if (wl->status_text != NULL)
+			xfree(wl->status_text);
+		memcpy(&wl->status_cell, &stdgc, sizeof wl->status_cell);
+		wl->status_text = status_print(c, wl, t, &wl->status_cell);
+		wl->status_width =
+		    screen_write_cstrlen(utf8flag, "%s", wl->status_text);
 
-	/* If there is enough space for the total width, all is gravy. */
-	if (width <= xx)
+		if (wl == s->curw)
+			wloffset = wlwidth;
+		wlwidth += wl->status_width + 1;
+	}
+
+	/* Create a new screen for the window list. */
+	screen_init(&window_list, wlwidth, 1, 0);
+
+	/* And draw the window list into it. */
+	screen_write_start(&ctx, NULL, &window_list);
+	RB_FOREACH(wl, winlinks, &s->windows) {
+		screen_write_cnputs(&ctx,
+		    -1, &wl->status_cell, utf8flag, "%s", wl->status_text);
+		screen_write_putc(&ctx, &stdgc, ' ');
+	}
+	screen_write_stop(&ctx);
+
+	/* If there is enough space for the total width, skip to draw now. */
+	if (wlwidth <= wlavailable)
 		goto draw;
 
 	/* Find size of current window text. */
-	size = status_width(s->curw);
+	wlsize = s->curw->status_width;
 
 	/*
-	 * If the offset is already on screen, we're good to draw from the
+	 * If the current window is already on screen, good to draw from the
 	 * start and just leave off the end.
 	 */
-	if (offset + size < xx) {
-		if (xx > 0) {
+	if (wloffset + wlsize < wlavailable) {
+		if (wlavailable > 0) {
 			rarrow = 1;
-			xx--;
+			wlavailable--;
+		}
+		wlwidth = wlavailable;
+	} else {
+		/*
+		 * Work out how many characters we need to omit from the
+		 * start. There are wlavailable characters to fill, and
+		 * wloffset + wlsize must be the last. So, the start character
+		 * is wloffset + wlsize - wlavailable.
+		 */
+		if (wlavailable > 0) {
+			larrow = 1;
+			wlavailable--;
 		}
 
-		width = xx;
-		goto draw;
+		wlstart = wloffset + wlsize - wlavailable;
+		if (wlavailable > 0 && wlwidth > wlstart + wlavailable + 1) {
+			rarrow = 1;
+			wlstart++;
+			wlavailable--;
+		}
+		wlwidth = wlavailable;
+	}
+
+	/* Bail if anything is now too small too. */
+	if (wlwidth == 0 || wlavailable == 0) {
+		screen_free(&window_list);
+		goto out;
 	}
 
 	/*
-	 * Work out how many characters we need to omit from the start. There
-	 * are xx characters to fill, and offset + size must be the last. So,
-	 * the start character is offset + size - xx.
+	 * Now the start position is known, work out the state of the left and
+	 * right arrows.
 	 */
-	if (xx > 0) {
-		larrow = 1;
-		xx--;
-	}
-
-	start = offset + size - xx;
- 	if (xx > 0 && width > start + xx + 1) { /* + 1, eh? */
- 		rarrow = 1;
- 		start++;
- 		xx--;
- 	}
- 	width = xx;
-
-draw:
-	/* Bail here if anything is too small too. XXX. */
-	if (width == 0 || xx == 0)
-		goto blank;
-
- 	/* Begin drawing and move to the starting position. */
-	screen_write_start(&ctx, NULL, &c->status);
-	if (llen != 0) {
- 		screen_write_cursormove(&ctx, 0, yy);
-		screen_write_cnputs(&ctx, llen, &sl_stdgc, utf8flag, "%s", left);
-		screen_write_putc(&ctx, &stdgc, ' ');
-		if (larrow)
-			screen_write_putc(&ctx, &stdgc, ' ');
-	} else {
-		if (larrow)
-			screen_write_cursormove(&ctx, 1, yy);
-		else
-			screen_write_cursormove(&ctx, 0, yy);
-	}
-
-	ox = 0;
-	if (width < xx) {
-		switch (options_get_number(&s->options, "status-justify")) {
-		case 1:	/* centered */
-			ox = 1 + (xx - width) / 2;
-			break;
-		case 2:	/* right */
-			ox = 1 + (xx - width);
-			break;
-		}
-		xx -= ox;
-		while (ox-- > 0)
-			screen_write_putc(&ctx, &stdgc, ' ');
-	}
-
-	/* Draw each character in succession. */
 	offset = 0;
 	RB_FOREACH(wl, winlinks, &s->windows) {
-		memcpy(&gc, &stdgc, sizeof gc);
-		text = status_print(s, wl, &gc);
+		if (wl->flags & WINLINK_ALERTFLAGS &&
+		    larrow == 1 && offset < wlstart)
+			larrow = -1;
 
-		if (larrow == 1 && offset < start) {
-			if (session_alert_has(s, wl, WINDOW_ACTIVITY))
-				larrow = -1;
-			else if (session_alert_has(s, wl, WINDOW_BELL))
-				larrow = -1;
-			else if (session_alert_has(s, wl, WINDOW_CONTENT))
-				larrow = -1;
-		}
+		offset += wl->status_width;
 
- 		for (ptr = text; *ptr != '\0'; ptr++) {
-			if (offset >= start && offset < start + width)
-				screen_write_putc(&ctx, &gc, *ptr);
-			offset++;
-		}
-
-		if (rarrow == 1 && offset > start + width) {
-			if (session_alert_has(s, wl, WINDOW_ACTIVITY))
-				rarrow = -1;
-			else if (session_alert_has(s, wl, WINDOW_BELL))
-				rarrow = -1;
-			else if (session_alert_has(s, wl, WINDOW_CONTENT))
-				rarrow = -1;
-		}
-
-		if (offset < start + width) {
-			if (offset >= start) {
-				screen_write_putc(&ctx, &stdgc, ' ');
-			}
-			offset++;
-		}
-
-		xfree(text);
+		if (wl->flags & WINLINK_ALERTFLAGS &&
+		    rarrow == 1 && offset > wlstart + wlwidth)
+			rarrow = -1;
 	}
 
-	/* Fill the remaining space if any. */
- 	while (offset++ < xx)
-		screen_write_putc(&ctx, &stdgc, ' ');
+draw:
+	/* Begin drawing. */
+	screen_write_start(&ctx, NULL, &c->status);
 
-	/* Draw the last item. */
-	if (rlen != 0) {
-		screen_write_cursormove(&ctx, c->tty.sx - rlen - 1, yy);
+	/* Draw the left string and arrow. */
+	screen_write_cursormove(&ctx, 0, 0);
+	if (llen != 0) {
+		screen_write_cnputs(&ctx, llen, &lgc, utf8flag, "%s", left);
 		screen_write_putc(&ctx, &stdgc, ' ');
-		screen_write_cnputs(&ctx, rlen, &sr_stdgc, utf8flag, "%s", right);
 	}
-
-	/* Draw the arrows. */
 	if (larrow != 0) {
 		memcpy(&gc, &stdgc, sizeof gc);
 		if (larrow == -1)
 			gc.attr ^= GRID_ATTR_REVERSE;
-		if (llen != 0)
-			screen_write_cursormove(&ctx, llen + 1, yy);
-		else
-			screen_write_cursormove(&ctx, 0, yy);
 		screen_write_putc(&ctx, &gc, '<');
 	}
+
+	/* Draw the right string and arrow. */
 	if (rarrow != 0) {
+		screen_write_cursormove(&ctx, c->tty.sx - rlen - 2, 0);
 		memcpy(&gc, &stdgc, sizeof gc);
 		if (rarrow == -1)
 			gc.attr ^= GRID_ATTR_REVERSE;
-		if (rlen != 0)
-			screen_write_cursormove(&ctx, c->tty.sx - rlen - 2, yy);
-		else
-			screen_write_cursormove(&ctx, c->tty.sx - 1, yy);
 		screen_write_putc(&ctx, &gc, '>');
+	} else
+		screen_write_cursormove(&ctx, c->tty.sx - rlen - 1, 0);
+	if (rlen != 0) {
+		screen_write_putc(&ctx, &stdgc, ' ');
+		screen_write_cnputs(&ctx, rlen, &rgc, utf8flag, "%s", right);
 	}
 
-	goto out;
+	/* Figure out the offset for the window list. */
+	if (llen != 0)
+		wloffset = llen + 1;
+	else
+		wloffset = 0;
+	if (wlwidth < wlavailable) {
+		switch (options_get_number(&s->options, "status-justify")) {
+		case 1:	/* centered */
+			wloffset += (wlavailable - wlwidth) / 2;
+			break;
+		case 2:	/* right */
+			wloffset += (wlavailable - wlwidth);
+			break;
+		}
+	}
+	if (larrow != 0)
+		wloffset++;
 
-blank:
- 	/* Just draw the whole line as blank. */
-	screen_write_start(&ctx, NULL, &c->status);
-	screen_write_cursormove(&ctx, 0, yy);
-	for (offset = 0; offset < c->tty.sx; offset++)
-		screen_write_putc(&ctx, &stdgc, ' ');
+	/* Copy the window list. */
+	screen_write_cursormove(&ctx, wloffset, 0);
+	screen_write_copy(&ctx, &window_list, wlstart, 0, wlwidth, 1);
+	screen_free(&window_list);
 
-out:
 	screen_write_stop(&ctx);
 
+out:
 	if (left != NULL)
 		xfree(left);
 	if (right != NULL)
@@ -317,125 +329,151 @@ out:
 	return (1);
 }
 
-char *
-status_replace(struct client *c, const char *fmt, time_t t)
+/* Replace a single special sequence (prefixed by #). */
+void
+status_replace1(struct client *c,struct winlink *wl,
+    char **iptr, char **optr, char *out, size_t outsize, int jobsflag)
 {
 	struct session *s = c->session;
-	struct winlink *wl = s->curw;
+	char		ch, tmp[256], *ptr, *endptr, *freeptr;
+	size_t		ptrlen;
+	long		limit;
+
+	if (wl == NULL)
+		wl = s->curw;
+
+	errno = 0;
+	limit = strtol(*iptr, &endptr, 10);
+	if ((limit == 0 && errno != EINVAL) ||
+	    (limit == LONG_MIN && errno != ERANGE) ||
+	    (limit == LONG_MAX && errno != ERANGE) ||
+	    limit != 0)
+		*iptr = endptr;
+	if (limit <= 0)
+		limit = LONG_MAX;
+
+	freeptr = NULL;
+
+	switch (*(*iptr)++) {
+	case '(':
+		if (!jobsflag) {
+			ch = ')';
+			goto skip_to;
+		}
+		if ((ptr = status_job(c, iptr)) == NULL)
+			return;
+		freeptr = ptr;
+		goto do_replace;
+	case 'H':
+		if (gethostname(tmp, sizeof tmp) != 0)
+			fatal("gethostname failed");
+		ptr = tmp;
+		goto do_replace;
+	case 'I':
+		xsnprintf(tmp, sizeof tmp, "%d", wl->idx);
+		ptr = tmp;
+		goto do_replace;
+	case 'P':
+		xsnprintf(tmp, sizeof tmp, "%u",
+		    window_pane_index(wl->window, wl->window->active));
+		ptr = tmp;
+		goto do_replace;
+	case 'S':
+		ptr = s->name;
+		goto do_replace;
+	case 'T':
+		ptr = wl->window->active->base.title;
+		goto do_replace;
+	case 'W':
+		ptr = wl->window->name;
+		goto do_replace;
+	case 'F':
+		tmp[0] = ' ';
+		if (wl->flags & WINLINK_CONTENT)
+			tmp[0] = '+';
+		else if (wl->flags & WINLINK_BELL)
+			tmp[0] = '!';
+		else if (wl->flags & WINLINK_ACTIVITY)
+			tmp[0] = '#';
+		else if (wl == s->curw)
+			tmp[0] = '*';
+		else if (wl == TAILQ_FIRST(&s->lastw))
+			tmp[0] = '-';
+		tmp[1] = '\0';
+		ptr = tmp;
+		goto do_replace;
+	case '[':
+		/*
+		 * Embedded style, handled at display time. Leave present and
+		 * skip input until ].
+		 */
+		ch = ']';
+		goto skip_to;
+	case '#':
+		*(*optr)++ = '#';
+		break;
+	}
+
+	return;
+
+do_replace:
+	ptrlen = strlen(ptr);
+	if ((size_t) limit < ptrlen)
+		ptrlen = limit;
+
+	if (*optr + ptrlen >= out + outsize - 1)
+		return;
+	while (ptrlen > 0 && *ptr != '\0') {
+		*(*optr)++ = *ptr++;
+		ptrlen--;
+	}
+
+	if (freeptr != NULL)
+		xfree(freeptr);
+	return;
+
+skip_to:
+	*(*optr)++ = '#';
+
+	(*iptr)--;	/* include ch */
+	while (**iptr != ch && **iptr != '\0') {
+		if (*optr >=  out + outsize - 1)
+			break;
+		*(*optr)++ = *(*iptr)++;
+	}
+}
+
+/* Replace special sequences in fmt. */
+char *
+status_replace(struct client *c,
+    struct winlink *wl, const char *fmt, time_t t, int jobsflag)
+{
 	static char	out[BUFSIZ];
-	char		in[BUFSIZ], tmp[256], ch, *iptr, *optr, *ptr, *endptr;
-	char           *savedptr;	/* freed at end of each loop */
-	size_t		len;
-	long		n;
-	
-	
+	char		in[BUFSIZ], ch, *iptr, *optr;
+
 	strftime(in, sizeof in, fmt, localtime(&t));
 	in[(sizeof in) - 1] = '\0';
 
 	iptr = in;
 	optr = out;
-	savedptr = NULL;
 
 	while (*iptr != '\0') {
 		if (optr >= out + (sizeof out) - 1)
 			break;
-		switch (ch = *iptr++) {
-		case '#':
-			errno = 0;
-			n = strtol(iptr, &endptr, 10);
-			if ((n == 0 && errno != EINVAL) ||
-			    (n == LONG_MIN && errno != ERANGE) ||
-			    (n == LONG_MAX && errno != ERANGE) ||
-			    n != 0)
-				iptr = endptr;
-			if (n <= 0)
-				n = LONG_MAX;
+		ch = *iptr++;
 
-			ptr = NULL;
-			switch (*iptr++) {
-			case '(':
-				if (ptr == NULL) {
-					ptr = status_job(c, &iptr);
-					if (ptr == NULL)
-						break;
-					savedptr = ptr;
-				}
-				/* FALLTHROUGH */
-			case 'H':
-				if (ptr == NULL) {
-					if (gethostname(tmp, sizeof tmp) != 0)
-						fatal("gethostname failed");
-					ptr = tmp;
-				}
-				/* FALLTHROUGH */
-			case 'I':
-				if (ptr == NULL) {
-					xsnprintf(tmp, sizeof tmp, "%d", wl->idx);
-					ptr = tmp;
-				}
-				/* FALLTHROUGH */
-			case 'P':
-				if (ptr == NULL) {
-					xsnprintf(tmp, sizeof tmp, "%u",
-					    window_pane_index(wl->window,
-					    wl->window->active));
-					ptr = tmp;
-				}
-				/* FALLTHROUGH */
-			case 'S':
-				if (ptr == NULL)
-					ptr = s->name;
-				/* FALLTHROUGH */
-			case 'T':
-				if (ptr == NULL)
-					ptr = wl->window->active->base.title;
-				/* FALLTHROUGH */
-			case 'W':
-				if (ptr == NULL)
-					ptr = wl->window->name;
-				len = strlen(ptr);
-				if ((size_t) n < len)
-					len = n;
-				if (optr + len >= out + (sizeof out) - 1)
-					break;
-				while (len > 0 && *ptr != '\0') {
-					*optr++ = *ptr++;
-					len--;
-				}
-				break;
-			case '[':
-				/* 
-				 * Embedded style, handled at display time.
-				 * Leave present and skip input until ].
-				 */
-				*optr++ = '#';
-
-				iptr--;	/* include [ */
-				while (*iptr != ']' && *iptr != '\0') {
-					if (optr >= out + (sizeof out) - 1)
-						break;
-					*optr++ = *iptr++;
-				}
-				break;
-			case '#':
-				*optr++ = '#';
-				break;
-			}
-			if (savedptr != NULL) {
-				xfree(savedptr);
-				savedptr = NULL;
-			}
-			break;
-		default:
+		if (ch != '#') {
 			*optr++ = ch;
-			break;
+			continue;
 		}
+		status_replace1(c, wl, &iptr, &optr, out, sizeof out, jobsflag);
 	}
 	*optr = '\0';
 
 	return (xstrdup(out));
 }
 
+/* Figure out job name and get its result, starting it off if necessary. */
 char *
 status_job(struct client *c, char **iptr)
 {
@@ -478,44 +516,48 @@ status_job(struct client *c, char **iptr)
 		    JOB_PERSIST, c, cmd, status_job_callback, xfree, NULL);
 		job_run(job);
 	}
+	xfree(cmd);
 	if (job->data == NULL)
 		return (xstrdup(""));
 	return (xstrdup(job->data));
 }
 
+/* Job has finished: save its result. */
 void
 status_job_callback(struct job *job)
 {
-	char	*buf;
+	char	*line, *buf;
 	size_t	 len;
 
-	len = BUFFER_USED(job->out);
-	buf = xmalloc(len + 1);
-	if (len != 0)
-		buffer_read(job->out, buf, len);
-	buf[len] = '\0';
-	buf[strcspn(buf, "\n")] = '\0';
+	buf = NULL;
+	if ((line = evbuffer_readline(job->event->input)) == NULL) {
+		len = EVBUFFER_LENGTH(job->event->input);
+		buf = xmalloc(len + 1);
+		if (len != 0)
+			memcpy(buf, EVBUFFER_DATA(job->event->input), len);
+		buf[len] = '\0';
+	}
 
 	if (job->data != NULL)
 		xfree(job->data);
 	else
 		server_redraw_client(job->client);
-	job->data = xstrdup(buf);
 
-	xfree(buf);
-}
-
-size_t
-status_width(struct winlink *wl)
-{
-	return (xsnprintf(NULL, 0, "%d:%s ", wl->idx, wl->window->name));
+	if (line == NULL)
+		job->data = buf;
+	else
+		job->data = xstrdup(line);
 }
 
+/* Return winlink status line entry and adjust gc as necessary. */
 char *
-status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
+status_print(
+    struct client *c, struct winlink *wl, time_t t, struct grid_cell *gc)
 {
 	struct options	*oo = &wl->window->options;
-	char   		*text, flag;
+	struct session	*s = c->session;
+	const char	*fmt;
+	char   		*text;
 	u_char		 fg, bg, attr;
 
 	fg = options_get_number(oo, "window-status-fg");
@@ -527,10 +569,7 @@ status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
 	attr = options_get_number(oo, "window-status-attr");
 	if (attr != 0)
 		gc->attr = attr;
-
-	flag = ' ';
- 	if (wl == TAILQ_FIRST(&s->lastw))
-		flag = '-';
+	fmt = options_get_string(oo, "window-status-format");
 	if (wl == s->curw) {
 		fg = options_get_number(oo, "window-status-current-fg");
 		if (fg != 8)
@@ -541,30 +580,35 @@ status_print(struct session *s, struct winlink *wl, struct grid_cell *gc)
 		attr = options_get_number(oo, "window-status-current-attr");
 		if (attr != 0)
 			gc->attr = attr;
-		flag = '*';
+		fmt = options_get_string(oo, "window-status-current-format");
 	}
 
-	if (session_alert_has(s, wl, WINDOW_ACTIVITY)) {
-		flag = '#';
-		gc->attr ^= GRID_ATTR_REVERSE;
-	} else if (session_alert_has(s, wl, WINDOW_BELL)) {
-		flag = '!';
-		gc->attr ^= GRID_ATTR_REVERSE;
-	} else if (session_alert_has(s, wl, WINDOW_CONTENT)) {
-		flag = '+';
-		gc->attr ^= GRID_ATTR_REVERSE;
+	if (wl->flags & WINLINK_ALERTFLAGS) {
+		fg = options_get_number(oo, "window-status-alert-fg");
+		if (fg != 8)
+			colour_set_fg(gc, fg);
+		bg = options_get_number(oo, "window-status-alert-bg");
+		if (bg != 8)
+			colour_set_bg(gc, bg);
+		attr = options_get_number(oo, "window-status-alert-attr");
+		if (attr != 0)
+			gc->attr = attr;
 	}
 
-	xasprintf(&text, "%d:%s%c", wl->idx, wl->window->name, flag);
+	text = status_replace(c, wl, fmt, t, 1);
 	return (text);
 }
 
+/* Set a status line message. */
 void printflike2
 status_message_set(struct client *c, const char *fmt, ...)
 {
-	struct timeval	tv;
-	va_list		ap;
-	int		delay;
+	struct timeval		 tv;
+	struct session		*s = c->session;
+	struct message_entry	*msg;
+	va_list			 ap;
+	int			 delay;
+	u_int			 i, limit;
 
 	status_prompt_clear(c);
 	status_message_clear(c);
@@ -573,18 +617,37 @@ status_message_set(struct client *c, const char *fmt, ...)
 	xvasprintf(&c->message_string, fmt, ap);
 	va_end(ap);
 
+	ARRAY_EXPAND(&c->message_log, 1);
+	msg = &ARRAY_LAST(&c->message_log);
+	msg->msg_time = time(NULL);
+	msg->msg = xstrdup(c->message_string);
+
+	if (s == NULL)
+		limit = 0;
+	else
+		limit = options_get_number(&s->options, "message-limit");
+	if (ARRAY_LENGTH(&c->message_log) > limit) {
+		limit = ARRAY_LENGTH(&c->message_log) - limit;
+		for (i = 0; i < limit; i++) {
+			msg = &ARRAY_FIRST(&c->message_log);
+			xfree(msg->msg);
+			ARRAY_REMOVE(&c->message_log, 0);
+		}
+	}
+
 	delay = options_get_number(&c->session->options, "display-time");
 	tv.tv_sec = delay / 1000;
 	tv.tv_usec = (delay % 1000) * 1000L;
 
-	if (gettimeofday(&c->message_timer, NULL) != 0)
-		fatal("gettimeofday failed");
-	timeradd(&c->message_timer, &tv, &c->message_timer);
+	evtimer_del(&c->message_timer);
+	evtimer_set(&c->message_timer, status_message_callback, c);
+	evtimer_add(&c->message_timer, &tv);
 
 	c->tty.flags |= (TTY_NOCURSOR|TTY_FREEZE);
 	c->flags |= CLIENT_STATUS;
 }
 
+/* Clear status line message. */
 void
 status_message_clear(struct client *c)
 {
@@ -600,6 +663,16 @@ status_message_clear(struct client *c)
 	screen_reinit(&c->status);
 }
 
+/* Clear status line message after timer expires. */
+/* ARGSUSED */
+void
+status_message_callback(unused int fd, unused short event, void *data)
+{
+	struct client	*c = data;
+
+	status_message_clear(c);
+}
+
 /* Draw client message on status line of present else on last line. */
 int
 status_message_redraw(struct client *c)
@@ -609,13 +682,16 @@ status_message_redraw(struct client *c)
 	struct screen		        old_status;
 	size_t			        len;
 	struct grid_cell		gc;
+	int				utf8flag;
 
 	if (c->tty.sx == 0 || c->tty.sy == 0)
 		return (0);
 	memcpy(&old_status, &c->status, sizeof old_status);
 	screen_init(&c->status, c->tty.sx, 1, 0);
 
-	len = strlen(c->message_string);
+	utf8flag = options_get_number(&s->options, "status-utf8");
+
+	len = screen_write_strlen(utf8flag, "%s", c->message_string);
 	if (len > c->tty.sx)
 		len = c->tty.sx;
 
@@ -627,7 +703,7 @@ status_message_redraw(struct client *c)
 	screen_write_start(&ctx, NULL, &c->status);
 
 	screen_write_cursormove(&ctx, 0, 0);
-	screen_write_puts(&ctx, &gc, "%.*s", (int) len, c->message_string);
+	screen_write_nputs(&ctx, len, &gc, utf8flag, "%s", c->message_string);
 	for (; len < c->tty.sx; len++)
 		screen_write_putc(&ctx, &gc, ' ');
 
@@ -641,6 +717,7 @@ status_message_redraw(struct client *c)
 	return (1);
 }
 
+/* Enable status line prompt. */
 void
 status_prompt_set(struct client *c, const char *msg,
     int (*callbackfn)(void *, const char *), void (*freefn)(void *),
@@ -674,10 +751,11 @@ status_prompt_set(struct client *c, const char *msg,
 	c->flags |= CLIENT_STATUS;
 }
 
+/* Remove status line prompt. */
 void
 status_prompt_clear(struct client *c)
 {
- 	if (c->prompt_string == NULL)
+	if (c->prompt_string == NULL)
 		return;
 
 	if (c->prompt_freefn != NULL && c->prompt_data != NULL)
@@ -695,6 +773,7 @@ status_prompt_clear(struct client *c)
 	screen_reinit(&c->status);
 }
 
+/* Update status line prompt with a new prompt string. */
 void
 status_prompt_update(struct client *c, const char *msg)
 {
@@ -713,22 +792,24 @@ status_prompt_update(struct client *c, const char *msg)
 int
 status_prompt_redraw(struct client *c)
 {
-	struct screen_write_ctx	ctx;
+	struct screen_write_ctx		ctx;
 	struct session		       *s = c->session;
 	struct screen		        old_status;
 	size_t			        i, size, left, len, off;
-	char				ch;
-	struct grid_cell		gc;
+	struct grid_cell		gc, *gcp;
+	int				utf8flag;
 
 	if (c->tty.sx == 0 || c->tty.sy == 0)
 		return (0);
 	memcpy(&old_status, &c->status, sizeof old_status);
 	screen_init(&c->status, c->tty.sx, 1, 0);
-	off = 0;
 
-	len = strlen(c->prompt_string);
+	utf8flag = options_get_number(&s->options, "status-utf8");
+
+	len = screen_write_strlen(utf8flag, "%s", c->prompt_string);
 	if (len > c->tty.sx)
 		len = c->tty.sx;
+	off = 0;
 
 	memcpy(&gc, &grid_default_cell, sizeof gc);
 	colour_set_fg(&gc, options_get_number(&s->options, "message-fg"));
@@ -738,35 +819,31 @@ status_prompt_redraw(struct client *c)
 	screen_write_start(&ctx, NULL, &c->status);
 
 	screen_write_cursormove(&ctx, 0, 0);
-	screen_write_puts(&ctx, &gc, "%.*s", (int) len, c->prompt_string);
+	screen_write_nputs(&ctx, len, &gc, utf8flag, "%s", c->prompt_string);
 
 	left = c->tty.sx - len;
 	if (left != 0) {
-		if (c->prompt_index < left)
-			size = strlen(c->prompt_buffer);
-		else {
+		size = screen_write_strlen(utf8flag, "%s", c->prompt_buffer);
+		if (c->prompt_index >= left) {
 			off = c->prompt_index - left + 1;
-			if (c->prompt_index == strlen(c->prompt_buffer))
+			if (c->prompt_index == size)
 				left--;
 			size = left;
 		}
-		screen_write_puts(
-		    &ctx, &gc, "%.*s", (int) left, c->prompt_buffer + off);
+		screen_write_nputs(
+		    &ctx, left, &gc, utf8flag, "%s", c->prompt_buffer + off);
 
 		for (i = len + size; i < c->tty.sx; i++)
 			screen_write_putc(&ctx, &gc, ' ');
-
-		/* Draw a fake cursor. */
-		ch = ' ';
-		screen_write_cursormove(&ctx, len + c->prompt_index - off, 0);
-		if (c->prompt_index < strlen(c->prompt_buffer))
-			ch = c->prompt_buffer[c->prompt_index];
-		gc.attr ^= GRID_ATTR_REVERSE;
-		screen_write_putc(&ctx, &gc, ch);
 	}
 
 	screen_write_stop(&ctx);
 
+	/* Apply fake cursor. */
+	off = len + c->prompt_index - off;
+	gcp = grid_view_get_cell(c->status.grid, off, 0);
+	gcp->attr ^= GRID_ATTR_REVERSE;
+
 	if (grid_compare(c->status.grid, old_status.grid) == 0) {
 		screen_free(&old_status);
 		return (0);
@@ -781,6 +858,7 @@ status_prompt_key(struct client *c, int key)
 {
 	struct paste_buffer	*pb;
 	char   			*s, *first, *last, word[64], swapc;
+	u_char			 ch;
 	size_t			 size, n, off, idx;
 
 	size = strlen(c->prompt_buffer);
@@ -847,12 +925,12 @@ status_prompt_key(struct client *c, int key)
 		size -= last - first;
 
 		/* Insert the new word. */
- 		size += strlen(s);
+		size += strlen(s);
 		off = first - c->prompt_buffer;
- 		c->prompt_buffer = xrealloc(c->prompt_buffer, 1, size + 1);
+		c->prompt_buffer = xrealloc(c->prompt_buffer, 1, size + 1);
 		first = c->prompt_buffer + off;
- 		memmove(first + strlen(s), first, n);
- 		memcpy(first, s, strlen(s));
+		memmove(first + strlen(s), first, n);
+		memcpy(first, s, strlen(s));
 
 		c->prompt_index = (first - c->prompt_buffer) + strlen(s);
 		xfree(s);
@@ -872,7 +950,7 @@ status_prompt_key(struct client *c, int key)
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
- 	case MODEKEYEDIT_DELETE:
+	case MODEKEYEDIT_DELETE:
 		if (c->prompt_index != size) {
 			memmove(c->prompt_buffer + c->prompt_index,
 			    c->prompt_buffer + c->prompt_index + 1,
@@ -922,7 +1000,8 @@ status_prompt_key(struct client *c, int key)
 		if ((pb = paste_get_top(&c->session->buffers)) == NULL)
 			break;
 		for (n = 0; n < pb->size; n++) {
-			if (pb->data[n] < 32 || pb->data[n] == 127)
+			ch = (u_char) pb->data[n];
+			if (ch < 32 || ch == 127)
 				break;
 		}
 
@@ -953,7 +1032,7 @@ status_prompt_key(struct client *c, int key)
 			c->flags |= CLIENT_STATUS;
 		}
 		break;
- 	case MODEKEYEDIT_ENTER:
+	case MODEKEYEDIT_ENTER:
 		if (*c->prompt_buffer != '\0')
 			status_prompt_add_history(c);
 		if (c->prompt_callbackfn(c->prompt_data, c->prompt_buffer) == 0)
@@ -964,7 +1043,7 @@ status_prompt_key(struct client *c, int key)
 			status_prompt_clear(c);
 		break;
 	case MODEKEY_OTHER:
-		if (key < 32 || key > 126)
+		if ((key & 0xff00) != 0 || key < 32 || key == 127)
 			break;
 		c->prompt_buffer = xrealloc(c->prompt_buffer, 1, size + 2);
 
@@ -1012,7 +1091,7 @@ char *
 status_prompt_complete(const char *s)
 {
 	const struct cmd_entry 	      **cmdent;
-	const struct set_option_entry  *optent;
+	const struct set_option_entry  *entry;
 	ARRAY_DECL(, const char *)	list;
 	char			       *prefix, *s2;
 	u_int				i;
@@ -1027,13 +1106,17 @@ status_prompt_complete(const char *s)
 		if (strncmp((*cmdent)->name, s, strlen(s)) == 0)
 			ARRAY_ADD(&list, (*cmdent)->name);
 	}
-	for (optent = set_option_table; optent->name != NULL; optent++) {
-		if (strncmp(optent->name, s, strlen(s)) == 0)
-			ARRAY_ADD(&list, optent->name);
+	for (entry = set_option_table; entry->name != NULL; entry++) {
+		if (strncmp(entry->name, s, strlen(s)) == 0)
+			ARRAY_ADD(&list, entry->name);
 	}
-	for (optent = set_window_option_table; optent->name != NULL; optent++) {
-		if (strncmp(optent->name, s, strlen(s)) == 0)
-			ARRAY_ADD(&list, optent->name);
+	for (entry = set_session_option_table; entry->name != NULL; entry++) {
+		if (strncmp(entry->name, s, strlen(s)) == 0)
+			ARRAY_ADD(&list, entry->name);
+	}
+	for (entry = set_window_option_table; entry->name != NULL; entry++) {
+		if (strncmp(entry->name, s, strlen(s)) == 0)
+			ARRAY_ADD(&list, entry->name);
 	}
 
 	/* If none, bail now. */
