@@ -1,4 +1,4 @@
-/* $Id: window-choose.c,v 1.24 2009/10/12 00:18:19 tcunha Exp $ */
+/* $Id: window-choose.c,v 1.30 2010/05/22 21:56:04 micahcowan Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -25,13 +25,13 @@
 struct screen *window_choose_init(struct window_pane *);
 void	window_choose_free(struct window_pane *);
 void	window_choose_resize(struct window_pane *, u_int, u_int);
-void	window_choose_key(struct window_pane *, struct client *, int);
+void	window_choose_key(struct window_pane *, struct session *, int);
 void	window_choose_mouse(
-    	    struct window_pane *, struct client *, struct mouse_event *);
+	    struct window_pane *, struct session *, struct mouse_event *);
 
 void	window_choose_redraw_screen(struct window_pane *);
 void	window_choose_write_line(
-    	    struct window_pane *, struct screen_write_ctx *, u_int);
+	    struct window_pane *, struct screen_write_ctx *, u_int);
 
 void	window_choose_scroll_up(struct window_pane *);
 void	window_choose_scroll_down(struct window_pane *);
@@ -64,6 +64,9 @@ struct window_choose_mode_data {
 	void		       *data;
 };
 
+int	window_choose_key_index(struct window_choose_mode_data *, u_int);
+int	window_choose_index_key(struct window_choose_mode_data *, int);
+
 void
 window_choose_vadd(struct window_pane *wp, int idx, const char *fmt, va_list ap)
 {
@@ -166,14 +169,16 @@ window_choose_resize(struct window_pane *wp, u_int sx, u_int sy)
 	window_choose_redraw_screen(wp);
 }
 
+/* ARGSUSED */
 void
-window_choose_key(struct window_pane *wp, unused struct client *c, int key)
+window_choose_key(struct window_pane *wp, unused struct session *sess, int key)
 {
 	struct window_choose_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
 	struct screen_write_ctx		 ctx;
 	struct window_choose_mode_item	*item;
-	u_int				 items;
+	u_int		       		 items;
+	int				 idx;
 
 	items = ARRAY_LENGTH(&data->list);
 
@@ -220,16 +225,41 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 		}
 		data->selected++;
 
-		if (data->selected >= data->top + screen_size_y(&data->screen))
-			window_choose_scroll_down(wp);
-		else {
+		if (data->selected < data->top + screen_size_y(s)) {
 			screen_write_start(&ctx, wp, NULL);
 			window_choose_write_line(
 			    wp, &ctx, data->selected - data->top);
 			window_choose_write_line(
 			    wp, &ctx, data->selected - 1 - data->top);
 			screen_write_stop(&ctx);
-		}
+		} else
+			window_choose_scroll_down(wp);
+		break;
+	case MODEKEYCHOICE_SCROLLUP:
+		if (items == 0 || data->top == 0)
+			break;
+		if (data->selected == data->top + screen_size_y(s) - 1) {
+			data->selected--;
+			window_choose_scroll_up(wp);
+			screen_write_start(&ctx, wp, NULL);
+			window_choose_write_line(
+			    wp, &ctx, screen_size_y(s) - 1);
+			screen_write_stop(&ctx);
+		} else
+			window_choose_scroll_up(wp);
+		break;
+	case MODEKEYCHOICE_SCROLLDOWN:
+		if (items == 0 ||
+		    data->top + screen_size_y(&data->screen) >= items)
+			break;
+		if (data->selected == data->top) {
+			data->selected++;
+			window_choose_scroll_down(wp);
+			screen_write_start(&ctx, wp, NULL);
+			window_choose_write_line(wp, &ctx, 0);
+			screen_write_stop(&ctx);
+		} else
+			window_choose_scroll_down(wp);
 		break;
 	case MODEKEYCHOICE_PAGEUP:
 		if (data->selected < screen_size_y(s)) {
@@ -242,7 +272,7 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 			else
 				data->top -= screen_size_y(s);
 		}
- 		window_choose_redraw_screen(wp);
+		window_choose_redraw_screen(wp);
 		break;
 	case MODEKEYCHOICE_PAGEDOWN:
 		data->selected += screen_size_y(s);
@@ -259,13 +289,22 @@ window_choose_key(struct window_pane *wp, unused struct client *c, int key)
 		window_choose_redraw_screen(wp);
 		break;
 	default:
+		idx = window_choose_index_key(data, key);
+		if (idx < 0 || (u_int) idx >= ARRAY_LENGTH(&data->list))
+			break;
+		data->selected = idx;
+
+		item = &ARRAY_ITEM(&data->list, data->selected);
+		data->callbackfn(data->data, item->idx);
+		window_pane_reset_mode(wp);
 		break;
 	}
 }
 
+/* ARGSUSED */
 void
 window_choose_mouse(
-    struct window_pane *wp, unused struct client *c, struct mouse_event *m)
+    struct window_pane *wp, unused struct session *sess, struct mouse_event *m)
 {
 	struct window_choose_mode_data	*data = wp->modedata;
 	struct screen			*s = &data->screen;
@@ -298,7 +337,7 @@ window_choose_write_line(
 	struct options			*oo = &wp->window->options;
 	struct screen			*s = &data->screen;
 	struct grid_cell		 gc;
- 	int				 utf8flag;
+	int				 utf8flag, key;
 
 	if (data->callbackfn == NULL)
 		fatalx("called before callback assigned");
@@ -314,13 +353,56 @@ window_choose_write_line(
 	screen_write_cursormove(ctx, 0, py);
 	if (data->top + py  < ARRAY_LENGTH(&data->list)) {
 		item = &ARRAY_ITEM(&data->list, data->top + py);
-		screen_write_nputs(
-		    ctx, screen_size_x(s) - 1, &gc, utf8flag, "%s", item->name);
+		key = window_choose_key_index(data, data->top + py);
+		if (key != -1) {
+			screen_write_nputs(ctx, screen_size_x(s) - 1,
+			    &gc, utf8flag, "(%c) %s", key, item->name);
+		} else {
+			screen_write_nputs(ctx, screen_size_x(s) - 1,
+			    &gc, utf8flag, "    %s", item->name);
+		}
+
 	}
 	while (s->cx < screen_size_x(s))
 		screen_write_putc(ctx, &gc, ' ');
 }
 
+int
+window_choose_key_index(struct window_choose_mode_data *data, u_int idx)
+{
+	static const char	keys[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+	const char	       *ptr;
+	int			mkey;
+
+	for (ptr = keys; *ptr != '\0'; ptr++) {
+		mkey = mode_key_lookup(&data->mdata, *ptr);
+		if (mkey != MODEKEY_NONE && mkey != MODEKEY_OTHER)
+			continue;
+		if (idx-- == 0)
+			return (*ptr);
+	}
+	return (-1);
+}
+
+int
+window_choose_index_key(struct window_choose_mode_data *data, int key)
+{
+	static const char	keys[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+	const char	       *ptr;
+	int			mkey;
+	u_int			idx = 0;
+
+	for (ptr = keys; *ptr != '\0'; ptr++) {
+		mkey = mode_key_lookup(&data->mdata, *ptr);
+		if (mkey != MODEKEY_NONE && mkey != MODEKEY_OTHER)
+			continue;
+		if (key == *ptr)
+			return (idx);
+		idx++;
+	}
+	return (-1);
+}
+
 void
 window_choose_redraw_screen(struct window_pane *wp)
 {
