@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.117 2009/10/23 17:41:20 tcunha Exp $ */
+/* $Id: window.c,v 1.134 2010/07/17 14:38:13 tcunha Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -54,6 +54,9 @@
 /* Global window list. */
 struct windows windows;
 
+void	window_pane_read_callback(struct bufferevent *, void *);
+void	window_pane_error_callback(struct bufferevent *, short, void *);
+
 RB_GENERATE(winlinks, winlink, entry, winlink_cmp);
 
 int
@@ -144,6 +147,8 @@ winlink_remove(struct winlinks *wwl, struct winlink *wl)
 	struct window	*w = wl->window;
 
 	RB_REMOVE(winlinks, wwl, wl);
+	if (wl->status_text != NULL)
+		xfree(wl->status_text);
 	xfree(wl);
 
 	if (w->references == 0)
@@ -154,17 +159,39 @@ winlink_remove(struct winlinks *wwl, struct winlink *wl)
 }
 
 struct winlink *
-winlink_next(unused struct winlinks *wwl, struct winlink *wl)
+winlink_next(struct winlink *wl)
 {
 	return (RB_NEXT(winlinks, wwl, wl));
 }
 
 struct winlink *
-winlink_previous(unused struct winlinks *wwl, struct winlink *wl)
+winlink_previous(struct winlink *wl)
 {
 	return (RB_PREV(winlinks, wwl, wl));
 }
 
+struct winlink *
+winlink_next_by_number(struct winlink *wl, struct session *s, int n)
+{
+	for (; n > 0; n--) {
+		if ((wl = RB_NEXT(winlinks, wwl, wl)) == NULL)
+			wl = RB_MIN(winlinks, &s->windows);
+	}
+
+	return (wl);
+}
+
+struct winlink *
+winlink_previous_by_number(struct winlink *wl, struct session *s, int n)
+{
+	for (; n > 0; n--) {
+		if ((wl = RB_PREV(winlinks, wwl, wl)) == NULL)
+			wl = RB_MAX(winlinks, &s->windows);
+	}
+
+	return (wl);
+}
+
 void
 winlink_stack_push(struct winlink_stack *stack, struct winlink *wl)
 {
@@ -182,7 +209,7 @@ winlink_stack_remove(struct winlink_stack *stack, struct winlink *wl)
 
 	if (wl == NULL)
 		return;
-	
+
 	TAILQ_FOREACH(wl2, stack, sentry) {
 		if (wl2 == wl) {
 			TAILQ_REMOVE(stack, wl, sentry);
@@ -207,7 +234,7 @@ window_create1(u_int sx, u_int sy)
 	struct window	*w;
 	u_int		 i;
 
-	w = xmalloc(sizeof *w);
+	w = xcalloc(1, sizeof *w);
 	w->name = NULL;
 	w->flags = 0;
 
@@ -216,10 +243,12 @@ window_create1(u_int sx, u_int sy)
 
 	w->lastlayout = -1;
 	w->layout_root = NULL;
-	
+
 	w->sx = sx;
 	w->sy = sy;
 
+	queue_window_name(w);
+
 	options_init(&w->options, &global_w_options);
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
@@ -273,6 +302,8 @@ window_destroy(struct window *w)
 	if (w->layout_root != NULL)
 		layout_free(w);
 
+	evtimer_del(&w->name_timer);
+
 	options_free(&w->options);
 
 	window_destroy_panes(w);
@@ -358,6 +389,29 @@ window_pane_at_index(struct window *w, u_int idx)
 	return (NULL);
 }
 
+struct window_pane *
+window_pane_next_by_number(struct window *w, struct window_pane *wp, u_int n)
+{
+	for (; n > 0; n--) {
+		if ((wp = TAILQ_NEXT(wp, entry)) == NULL)
+			wp = TAILQ_FIRST(&w->panes);
+	}
+
+	return (wp);
+}
+
+struct window_pane *
+window_pane_previous_by_number(struct window *w, struct window_pane *wp,
+    u_int n)
+{
+	for (; n > 0; n--) {
+		if ((wp = TAILQ_PREV(wp, window_panes, entry)) == NULL)
+			wp = TAILQ_LAST(&w->panes, window_panes);
+	}
+
+	return (wp);
+}
+
 u_int
 window_pane_index(struct window *w, struct window_pane *wp)
 {
@@ -410,22 +464,21 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 	wp->cwd = NULL;
 
 	wp->fd = -1;
-	wp->in = buffer_create(BUFSIZ);
-	wp->out = buffer_create(BUFSIZ);
+	wp->event = NULL;
 
 	wp->mode = NULL;
 
 	wp->layout_cell = NULL;
 
 	wp->xoff = 0;
- 	wp->yoff = 0;
+	wp->yoff = 0;
 
 	wp->sx = sx;
 	wp->sy = sy;
 
 	wp->pipe_fd = -1;
-	wp->pipe_buf = NULL;
 	wp->pipe_off = 0;
+	wp->pipe_event = NULL;
 
 	wp->saved_grid = NULL;
 
@@ -440,8 +493,10 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 void
 window_pane_destroy(struct window_pane *wp)
 {
-	if (wp->fd != -1)
+	if (wp->fd != -1) {
 		close(wp->fd);
+		bufferevent_free(wp->event);
+	}
 
 	input_free(wp);
 
@@ -451,13 +506,10 @@ window_pane_destroy(struct window_pane *wp)
 		grid_destroy(wp->saved_grid);
 
 	if (wp->pipe_fd != -1) {
-		buffer_destroy(wp->pipe_buf);
 		close(wp->pipe_fd);
+		bufferevent_free(wp->pipe_event);
 	}
 
-	buffer_destroy(wp->in);
-	buffer_destroy(wp->out);
-
 	if (wp->cwd != NULL)
 		xfree(wp->cwd);
 	if (wp->shell != NULL)
@@ -471,18 +523,16 @@ int
 window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
     const char *cwd, struct environ *env, struct termios *tio, char **cause)
 {
-	struct winsize	 	 ws;
-	int		 	 mode;
-	char			*argv0, **varp, *var;
-	ARRAY_DECL(, char *)	 varlist;
-	struct environ_entry	*envent;
-	const char		*ptr;
-	struct timeval	 	 tv;
-	struct termios		 tio2;
-	u_int		 	 i;
+	struct winsize	 ws;
+	int		 mode;
+	char		*argv0;
+	const char	*ptr;
+	struct termios	 tio2;
 
-	if (wp->fd != -1)
+	if (wp->fd != -1) {
 		close(wp->fd);
+		bufferevent_free(wp->event);
+	}
 	if (cmd != NULL) {
 		if (wp->cmd != NULL)
 			xfree(wp->cmd);
@@ -503,13 +553,7 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 	ws.ws_col = screen_size_x(&wp->base);
 	ws.ws_row = screen_size_y(&wp->base);
 
-	if (gettimeofday(&wp->window->name_timer, NULL) != 0)
-		fatal("gettimeofday failed");
-	tv.tv_sec = 0;
-	tv.tv_usec = NAME_INTERVAL * 1000L;
-	timeradd(&wp->window->name_timer, &tv, &wp->window->name_timer);
-
- 	switch (wp->pid = forkpty(&wp->fd, wp->tty, NULL, &ws)) {
+	switch (wp->pid = forkpty(&wp->fd, wp->tty, NULL, &ws)) {
 	case -1:
 		wp->fd = -1;
 		xasprintf(cause, "%s: %s", cmd, strerror(errno));
@@ -526,22 +570,9 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		if (tcsetattr(STDIN_FILENO, TCSANOW, &tio2) != 0)
 			fatal("tcgetattr failed");
 
-		ARRAY_INIT(&varlist);
-		for (varp = environ; *varp != NULL; varp++) {
-			var = xstrdup(*varp);
-			var[strcspn(var, "=")] = '\0';
-			ARRAY_ADD(&varlist, var);
-		}
-		for (i = 0; i < ARRAY_LENGTH(&varlist); i++) {
-			var = ARRAY_ITEM(&varlist, i);
-			unsetenv(var);
-		}
-		RB_FOREACH(envent, environ, env) {
-			if (envent->value != NULL)
-				setenv(envent->name, envent->value, 1);
-		}
+		environ_push(env);
 
-		sigreset();
+		clear_signals();
 		log_close();
 
 		if (*wp->cmd != '\0') {
@@ -571,10 +602,42 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		fatal("fcntl failed");
 	if (fcntl(wp->fd, F_SETFD, FD_CLOEXEC) == -1)
 		fatal("fcntl failed");
+	wp->event = bufferevent_new(wp->fd,
+	    window_pane_read_callback, NULL, window_pane_error_callback, wp);
+	bufferevent_enable(wp->event, EV_READ|EV_WRITE);
 
 	return (0);
 }
 
+/* ARGSUSED */
+void
+window_pane_read_callback(unused struct bufferevent *bufev, void *data)
+{
+	struct window_pane     *wp = data;
+	char   		       *new_data;
+	size_t			new_size;
+
+	new_size = EVBUFFER_LENGTH(wp->event->input) - wp->pipe_off;
+	if (wp->pipe_fd != -1 && new_size > 0) {
+		new_data = EVBUFFER_DATA(wp->event->input);
+		bufferevent_write(wp->pipe_event, new_data, new_size);
+	}
+
+	input_parse(wp);
+
+	wp->pipe_off = EVBUFFER_LENGTH(wp->event->input);
+}
+
+/* ARGSUSED */
+void
+window_pane_error_callback(
+    unused struct bufferevent *bufev, unused short what, void *data)
+{
+	struct window_pane *wp = data;
+
+	server_destroy_pane(wp);
+}
+
 void
 window_pane_resize(struct window_pane *wp, u_int sx, u_int sy)
 {
@@ -606,6 +669,81 @@ window_pane_resize(struct window_pane *wp, u_int sx, u_int sy)
 		fatal("ioctl failed");
 }
 
+/*
+ * Enter alternative screen mode. A copy of the visible screen is saved and the
+ * history is not updated
+ */
+void
+window_pane_alternate_on(struct window_pane *wp, struct grid_cell *gc)
+{
+	struct screen	*s = &wp->base;
+	u_int		 sx, sy;
+
+	if (wp->saved_grid != NULL)
+		return;
+	if (!options_get_number(&wp->window->options, "alternate-screen"))
+		return;
+	sx = screen_size_x(s);
+	sy = screen_size_y(s);
+
+	wp->saved_grid = grid_create(sx, sy, 0);
+	grid_duplicate_lines(wp->saved_grid, 0, s->grid, screen_hsize(s), sy);
+	wp->saved_cx = s->cx;
+	wp->saved_cy = s->cy;
+	memcpy(&wp->saved_cell, gc, sizeof wp->saved_cell);
+
+	grid_view_clear(s->grid, 0, 0, sx, sy);
+
+	wp->base.grid->flags &= ~GRID_HISTORY;
+
+	wp->flags |= PANE_REDRAW;
+}
+
+/* Exit alternate screen mode and restore the copied grid. */
+void
+window_pane_alternate_off(struct window_pane *wp, struct grid_cell *gc)
+{
+	struct screen	*s = &wp->base;
+	u_int		 sx, sy;
+
+	if (wp->saved_grid == NULL)
+		return;
+	if (!options_get_number(&wp->window->options, "alternate-screen"))
+		return;
+	sx = screen_size_x(s);
+	sy = screen_size_y(s);
+
+	/*
+	 * If the current size is bigger, temporarily resize to the old size
+	 * before copying back.
+	 */
+	if (sy > wp->saved_grid->sy)
+		screen_resize(s, sx, wp->saved_grid->sy);
+
+	/* Restore the grid, cursor position and cell. */
+	grid_duplicate_lines(s->grid, screen_hsize(s), wp->saved_grid, 0, sy);
+	s->cx = wp->saved_cx;
+	if (s->cx > screen_size_x(s) - 1)
+		s->cx = screen_size_x(s) - 1;
+	s->cy = wp->saved_cy;
+	if (s->cy > screen_size_y(s) - 1)
+		s->cy = screen_size_y(s) - 1;
+	memcpy(gc, &wp->saved_cell, sizeof *gc);
+
+	/*
+	 * Turn history back on (so resize can use it) and then resize back to
+	 * the current size.
+	 */
+	wp->base.grid->flags |= GRID_HISTORY;
+	if (sy > wp->saved_grid->sy)
+		screen_resize(s, sx, sy);
+
+	grid_destroy(wp->saved_grid);
+	wp->saved_grid = NULL;
+
+	wp->flags |= PANE_REDRAW;
+}
+
 int
 window_pane_set_mode(struct window_pane *wp, const struct window_mode *mode)
 {
@@ -635,24 +773,7 @@ window_pane_reset_mode(struct window_pane *wp)
 }
 
 void
-window_pane_parse(struct window_pane *wp)
-{
-	size_t	new_size;
-
-	if (wp->mode != NULL)
-		return;
-
-	new_size = BUFFER_USED(wp->in) - wp->pipe_off;
-	if (wp->pipe_fd != -1 && new_size > 0)
-		buffer_write(wp->pipe_buf, BUFFER_OUT(wp->in), new_size);
-	
-	input_parse(wp);
-
-	wp->pipe_off = BUFFER_USED(wp->in);
-}
-
-void
-window_pane_key(struct window_pane *wp, struct client *c, int key)
+window_pane_key(struct window_pane *wp, struct session *sess, int key)
 {
 	struct window_pane	*wp2;
 
@@ -661,7 +782,7 @@ window_pane_key(struct window_pane *wp, struct client *c, int key)
 
 	if (wp->mode != NULL) {
 		if (wp->mode->key != NULL)
-			wp->mode->key(wp, c, key);
+			wp->mode->key(wp, sess, key);
 		return;
 	}
 
@@ -680,7 +801,7 @@ window_pane_key(struct window_pane *wp, struct client *c, int key)
 
 void
 window_pane_mouse(
-    struct window_pane *wp, struct client *c, struct mouse_event *m)
+    struct window_pane *wp, struct session *sess, struct mouse_event *m)
 {
 	if (!window_pane_visible(wp))
 		return;
@@ -694,7 +815,7 @@ window_pane_mouse(
 
 	if (wp->mode != NULL) {
 		if (wp->mode->mouse != NULL)
-			wp->mode->mouse(wp, c, m);
+			wp->mode->mouse(wp, sess, m);
 	} else if (wp->fd != -1)
 		input_mouse(wp, m);
 }
@@ -735,3 +856,113 @@ window_pane_search(struct window_pane *wp, const char *searchstr, u_int *lineno)
 	xfree(newsearchstr);
 	return (msg);
 }
+
+/* Find the pane directly above another. */
+struct window_pane *
+window_pane_find_up(struct window_pane *wp)
+{
+	struct window_pane     *wp2;
+	u_int			left, top;
+
+	if (wp == NULL || !window_pane_visible(wp))
+		return (NULL);
+
+	top = wp->yoff;
+	if (top == 0)
+		top = wp->window->sy + 1;
+	left = wp->xoff;
+
+	TAILQ_FOREACH(wp2, &wp->window->panes, entry) {
+		if (!window_pane_visible(wp2))
+			continue;
+		if (wp2->yoff + wp2->sy + 1 != top)
+			continue;
+		if (left >= wp2->xoff && left <= wp2->xoff + wp2->sx)
+			return (wp2);
+	}
+	return (NULL);
+}
+
+/* Find the pane directly below another. */
+struct window_pane *
+window_pane_find_down(struct window_pane *wp)
+{
+	struct window_pane     *wp2;
+	u_int			left, bottom;
+
+	if (wp == NULL || !window_pane_visible(wp))
+		return (NULL);
+
+	bottom = wp->yoff + wp->sy + 1;
+	if (bottom >= wp->window->sy)
+		bottom = 0;
+	left = wp->xoff;
+
+	TAILQ_FOREACH(wp2, &wp->window->panes, entry) {
+		if (!window_pane_visible(wp2))
+			continue;
+		if (wp2->yoff != bottom)
+			continue;
+		if (left >= wp2->xoff && left <= wp2->xoff + wp2->sx)
+			return (wp2);
+	}
+	return (NULL);
+}
+
+/*
+ * Find the pane directly to the left of another, adjacent to the left side and
+ * containing the top edge.
+ */
+struct window_pane *
+window_pane_find_left(struct window_pane *wp)
+{
+	struct window_pane     *wp2;
+	u_int			left, top;
+
+	if (wp == NULL || !window_pane_visible(wp))
+		return (NULL);
+
+	left = wp->xoff;
+	if (left == 0)
+		left = wp->window->sx + 1;
+	top = wp->yoff;
+
+	TAILQ_FOREACH(wp2, &wp->window->panes, entry) {
+		if (!window_pane_visible(wp2))
+			continue;
+		if (wp2->xoff + wp2->sx + 1 != left)
+			continue;
+		if (top >= wp2->yoff && top <= wp2->yoff + wp2->sy)
+			return (wp2);
+	}
+	return (NULL);
+}
+
+/*
+ * Find the pane directly to the right of another, that is adjacent to the
+ * right edge and including the top edge.
+ */
+struct window_pane *
+window_pane_find_right(struct window_pane *wp)
+{
+	struct window_pane     *wp2;
+	u_int			right, top;
+
+	if (wp == NULL || !window_pane_visible(wp))
+		return (NULL);
+
+	right = wp->xoff + wp->sx + 1;
+	if (right >= wp->window->sx)
+		right = 0;
+	top = wp->yoff;
+
+	TAILQ_FOREACH(wp2, &wp->window->panes, entry) {
+		if (!window_pane_visible(wp2))
+			continue;
+		if (wp2->xoff != right)
+			continue;
+		if (top >= wp2->yoff && top <= wp2->yoff + wp2->sy)
+			return (wp2);
+	}
+	return (NULL);
+}
