@@ -1,4 +1,4 @@
-/* $Id: xmalloc.c,v 1.8 2009/01/29 19:24:34 nicm Exp $ */
+/* $Id: xmalloc.c,v 1.12 2009/10/28 23:12:38 tcunha Exp $ */
 
 /*
  * Copyright (c) 2004 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -29,31 +29,29 @@
 char *
 xstrdup(const char *s)
 {
-	void	*ptr;
+	char	*ptr;
 	size_t	 len;
 
 	len = strlen(s) + 1;
 	ptr = xmalloc(len);
 
-        return (strncpy(ptr, s, len));
+	strlcpy(ptr, s, len);
+	return (ptr);
 }
 
 void *
 xcalloc(size_t nmemb, size_t size)
 {
-        void	*ptr;
+	void	*ptr;
 
-        if (size == 0 || nmemb == 0)
+	if (size == 0 || nmemb == 0)
 		fatalx("zero size");
-        if (SIZE_MAX / nmemb < size)
-                fatalx("nmemb * size > SIZE_MAX");
-        if ((ptr = calloc(nmemb, size)) == NULL)
+	if (SIZE_MAX / nmemb < size)
+		fatalx("nmemb * size > SIZE_MAX");
+	if ((ptr = calloc(nmemb, size)) == NULL)
 		fatal("xcalloc failed");
 
-#ifdef DEBUG
-	xmalloc_new(xmalloc_caller(), ptr, nmemb * size);
-#endif
-        return (ptr);
+	return (ptr);
 }
 
 void *
@@ -61,15 +59,12 @@ xmalloc(size_t size)
 {
 	void	*ptr;
 
-        if (size == 0)
-                fatalx("zero size");
-        if ((ptr = malloc(size)) == NULL)
+	if (size == 0)
+		fatalx("zero size");
+	if ((ptr = malloc(size)) == NULL)
 		fatal("xmalloc failed");
 
-#ifdef DEBUG
-	xmalloc_new(xmalloc_caller(), ptr, size);
-#endif
-        return (ptr);
+	return (ptr);
 }
 
 void *
@@ -79,16 +74,13 @@ xrealloc(void *oldptr, size_t nmemb, size_t size)
 	void	*newptr;
 
 	if (newsize == 0)
-                fatalx("zero size");
-        if (SIZE_MAX / nmemb < size)
-                fatalx("nmemb * size > SIZE_MAX");
-        if ((newptr = realloc(oldptr, newsize)) == NULL)
+		fatalx("zero size");
+	if (SIZE_MAX / nmemb < size)
+		fatalx("nmemb * size > SIZE_MAX");
+	if ((newptr = realloc(oldptr, newsize)) == NULL)
 		fatal("xrealloc failed");
 
-#ifdef DEBUG
-	xmalloc_change(xmalloc_caller(), oldptr, newptr, nmemb * size);
-#endif
-        return (newptr);
+	return (newptr);
 }
 
 void
@@ -97,21 +89,17 @@ xfree(void *ptr)
 	if (ptr == NULL)
 		fatalx("null pointer");
 	free(ptr);
-
-#ifdef DEBUG
-	xmalloc_free(ptr);
-#endif
 }
 
 int printflike2
 xasprintf(char **ret, const char *fmt, ...)
 {
-        va_list ap;
-        int	i;
+	va_list ap;
+	int	i;
 
-        va_start(ap, fmt);
-        i = xvasprintf(ret, fmt, ap);
-        va_end(ap);
+	va_start(ap, fmt);
+	i = xvasprintf(ret, fmt, ap);
+	va_end(ap);
 
 	return (i);
 }
@@ -122,24 +110,21 @@ xvasprintf(char **ret, const char *fmt, va_list ap)
 	int	i;
 
 	i = vasprintf(ret, fmt, ap);
-        if (i < 0 || *ret == NULL)
-                fatal("xvasprintf failed");
+	if (i < 0 || *ret == NULL)
+		fatal("xvasprintf failed");
 
-#ifdef DEBUG
-	xmalloc_new(xmalloc_caller(), *ret, i + 1);
-#endif
-        return (i);
+	return (i);
 }
 
 int printflike3
 xsnprintf(char *buf, size_t len, const char *fmt, ...)
 {
-        va_list ap;
-        int	i;
+	va_list ap;
+	int	i;
 
-        va_start(ap, fmt);
-        i = xvsnprintf(buf, len, fmt, ap);
-        va_end(ap);
+	va_start(ap, fmt);
+	i = xvsnprintf(buf, len, fmt, ap);
+	va_end(ap);
 
 	return (i);
 }
@@ -153,30 +138,8 @@ xvsnprintf(char *buf, size_t len, const char *fmt, va_list ap)
 		fatalx("len > INT_MAX");
 
 	i = vsnprintf(buf, len, fmt, ap);
-        if (i < 0)
-                fatal("vsnprintf failed");
+	if (i < 0)
+		fatal("vsnprintf failed");
 
-        return (i);
-}
-
-/*
- * Some systems modify the path in place. This function and xbasename below
- * avoid that by using a temporary buffer.
- */
-char *
-xdirname(const char *src)
-{
-	static char	dst[MAXPATHLEN];
-
-	strlcpy(dst, src, sizeof dst);
-	return (dirname(dst));
-}
-
-char *
-xbasename(const char *src)
-{
-	static char	dst[MAXPATHLEN];
-
-	strlcpy(dst, src, sizeof dst);
-	return (basename(dst));
+	return (i);
 }
