@@ -1,4 +1,4 @@
-/* $Id: xterm-keys.c,v 1.2 2009/10/28 23:05:43 tcunha Exp $ */
+/* $Id: xterm-keys.c,v 1.5 2009/12/04 22:14:47 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -27,11 +27,11 @@
  * character:
  *
  * 2 Shift
- * 3 Alt 
- * 4 Shift + Alt 
- * 5 Ctrl 
- * 6 Shift + Ctrl 
- * 7 Alt + Ctrl 
+ * 3 Alt
+ * 4 Shift + Alt
+ * 5 Ctrl
+ * 6 Shift + Ctrl
+ * 7 Alt + Ctrl
  * 8 Shift + Alt + Ctrl
  *
  * Rather than parsing them, just match against a table.
@@ -50,13 +50,13 @@ struct xterm_keys_entry {
 
 struct xterm_keys_entry xterm_keys_table[] = {
 	{ KEYC_F1,	"\033[1;_P" },
-	{ KEYC_F1,	"\033[O_P" },
+	{ KEYC_F1,	"\033O_P" },
 	{ KEYC_F2,	"\033[1;_Q" },
-	{ KEYC_F2,	"\033[O_Q" },
+	{ KEYC_F2,	"\033O_Q" },
 	{ KEYC_F3,	"\033[1;_R" },
-	{ KEYC_F3,	"\033[O_R" },
+	{ KEYC_F3,	"\033O_R" },
 	{ KEYC_F4,	"\033[1;_S" },
-	{ KEYC_F4,	"\033[O_S" },
+	{ KEYC_F4,	"\033O_S" },
 	{ KEYC_F5,	"\033[15;_~" },
 	{ KEYC_F6,	"\033[17;_~" },
 	{ KEYC_F7,	"\033[18;_~" },
@@ -85,22 +85,28 @@ struct xterm_keys_entry xterm_keys_table[] = {
 	{ KEYC_DC,	"\033[3;_~" },
 };
 
-/* Match key against buffer, treating _ as a wildcard. */
+/*
+ * Match key against buffer, treating _ as a wildcard. Return -1 for no match,
+ * 0 for match, 1 if the end of the buffer is reached (need more data).
+ */
 int
 xterm_keys_match(const char *template, const char *buf, size_t len)
 {
 	size_t	pos;
 
-	if (len == 0 || len < strlen(template))
+	if (len == 0)
 		return (0);
 
 	pos = 0;
 	do {
 		if (*template != '_' && buf[pos] != *template)
-			return (0);
+			return (-1);
 	} while (pos++ != len && *++template != '\0');
 
-	return (1);
+	if (*template != '\0')	/* partial */
+		return (1);
+
+	return (0);
 }
 
 /* Find modifiers based on template. */
@@ -126,27 +132,34 @@ xterm_keys_modifiers(const char *template, const char *buf, size_t len)
 	case '7':
 		return (KEYC_ESCAPE|KEYC_CTRL);
 	case '8':
- 		return (KEYC_SHIFT|KEYC_ESCAPE|KEYC_CTRL);
+		return (KEYC_SHIFT|KEYC_ESCAPE|KEYC_CTRL);
 	}
 	return (0);
 }
 
-/* Lookup key from buffer against table. */
+/*
+ * Lookup key from a buffer against the table. Returns 0 for found (and the
+ * key), -1 for not found, 1 for partial match.
+ */
 int
-xterm_keys_find(const char *buf, size_t len, size_t *size)
+xterm_keys_find(const char *buf, size_t len, size_t *size, int *key)
 {
 	struct xterm_keys_entry	*entry;
 	u_int			 i;
 
 	for (i = 0; i < nitems(xterm_keys_table); i++) {
 		entry = &xterm_keys_table[i];
-		if (xterm_keys_match(entry->template, buf, len))
-			break;
+		switch (xterm_keys_match(entry->template, buf, len)) {
+		case 0:
+			*size = strlen(entry->template);
+			*key = entry->key;
+			*key |= xterm_keys_modifiers(entry->template, buf, len);
+			return (0);
+		case 1:
+			return (1);
+		}
 	}
-	if (i == nitems(xterm_keys_table))
-		return (KEYC_NONE);
-	*size = strlen(entry->template);
-	return (entry->key | xterm_keys_modifiers(entry->template, buf, len));
+	return (-1);
 }
 
 /* Lookup a key number from the table. */
@@ -193,7 +206,7 @@ xterm_keys_lookup(int key)
 	}
 	if (i == nitems(xterm_keys_table))
 		return (NULL);
-	
+
 	/* Copy the template and replace the modifier. */
 	out = xstrdup(entry->template);
 	out[strcspn(out, "_")] = '0' + modifiers;
